@@ -10,6 +10,18 @@ templates_path = ['_templates']
 source_suffix = '.rst'
 master_doc = 'index'
 exclude_patterns = ['**/.#*', '**/*~', 'start/quick-common.rst']
+if tags.has('man'):
+    exclude_patterns += ['architecture.rst', 'glossary.rst', 'release*.rst',
+                         'api/*',
+                         'cephfs/*',
+                         'dev/*',
+                         'install/*',
+                         'mon/*',
+                         'rados/*',
+                         'radosgw/*',
+                         'rbd/*',
+                         'start/*']
+
 pygments_style = 'sphinx'
 
 html_theme = 'ceph'
@@ -27,9 +39,10 @@ extensions = [
     'sphinx.ext.autodoc',
     'sphinx.ext.graphviz',
     'sphinx.ext.todo',
-    'sphinx_ditaa',
+    'sphinxcontrib.ditaa',
     'breathe',
     ]
+ditaa = 'ditaa'
 todo_include_todos = True
 
 top_level = os.path.dirname(
