@@ -6,6 +6,7 @@ from cStringIO import StringIO
 import contextlib
 import logging
 import os
+import time
 
 from teuthology import misc as teuthology
 from teuthology import contextutil
@@ -335,7 +336,7 @@ def run_qemu(ctx, config):
         ceph_config = ctx.ceph['ceph'].conf.get('global', {})
         ceph_config.update(ctx.ceph['ceph'].conf.get('client', {}))
         ceph_config.update(ctx.ceph['ceph'].conf.get(client, {}))
-        if ceph_config.get('rbd cache'):
+        if ceph_config.get('rbd cache', True):
             if ceph_config.get('rbd cache max dirty', 1) > 0:
                 cachemode = 'writeback'
             else:
@@ -353,6 +354,7 @@ def run_qemu(ctx, config):
                     cachemode=cachemode,
                     ),
                 ])
+        time_wait = client_config.get('time_wait', 0)
 
         log.info('starting qemu...')
         procs.append(
@@ -370,6 +372,11 @@ def run_qemu(ctx, config):
         log.info('waiting for qemu tests to finish...')
         run.wait(procs)
 
+        if time_wait > 0:
+            log.debug('waiting {time_wait} sec for workloads detect finish...'.format(
+                time_wait=time_wait));
+            time.sleep(time_wait)
+
         log.debug('checking that qemu tests succeeded...')
         for client in config.iterkeys():
             (remote,) = ctx.cluster.only(client).remotes.keys()
