@@ -29,7 +29,7 @@ public:
   EntityName entity_name;
   uint64_t global_id;
 
-  AuthServiceHandler(CephContext *cct_) : cct(cct_), global_id(0) {}
+  explicit AuthServiceHandler(CephContext *cct_) : cct(cct_), global_id(0) {}
 
   virtual ~AuthServiceHandler() { }
 
@@ -39,7 +39,6 @@ public:
   EntityName& get_entity_name() { return entity_name; }
 };
 
-extern AuthServiceHandler *get_auth_service_handler(CephContext *cct,
-				KeyServer *ks, set<__u32>& supported);
+extern AuthServiceHandler *get_auth_service_handler(int type, CephContext *cct, KeyServer *ks);
 
 #endif
