@@ -17,16 +17,19 @@
 
 #include "../Auth.h"
 
+class CephContext;
+
 struct AuthNoneAuthorizer : public AuthAuthorizer {
   AuthNoneAuthorizer() : AuthAuthorizer(CEPH_AUTH_NONE) { }
-  bool build_authorizer(uint64_t global_id) {
+  bool build_authorizer(const EntityName &ename, uint64_t global_id) {
     __u8 struct_v = 1;
     ::encode(struct_v, bl);
-    ::encode(g_conf->name, bl);
+    ::encode(ename, bl);
     ::encode(global_id, bl);
     return 0;
   }
-  bool verify_reply(bufferlist::iterator& reply) { return true; }
+  bool verify_reply(bufferlist::iterator& reply) override { return true; }
+  bool add_challenge(CephContext *cct, bufferlist& ch) override { return true; }
 };
 
 #endif
