@@ -149,6 +149,16 @@ public:
     }
     do_queues();
   }
+
+  /**
+   * Has reservations
+   *
+   * Return true if there are reservations in progress
+   */
+  bool has_reservation() {
+    Mutex::Locker l(lock);
+    return !in_progress.empty();
+  }
   static const unsigned MAX_PRIORITY = (unsigned)-1;
 };
 
