@@ -15,6 +15,8 @@
 #ifndef CEPH_MEMORYMODEL_H
 #define CEPH_MEMORYMODEL_H
 
+class CephContext;
+
 class MemoryModel {
 public:
   struct snap {
@@ -27,15 +29,21 @@ public:
     
     int heap, malloc, mmap;
 
+    snap() : peak(0), size(0), hwm(0), rss(0), data(0), lib(0),
+	     heap(0), malloc(0), mmap(0)
+    {}
+
     int get_total() { return size; }
     int get_rss() { return rss; }
     int get_heap() { return heap; }
   } last;
 
 private:
+  CephContext *cct;
   void _sample(snap *p);
 
 public:
+  explicit MemoryModel(CephContext *cct);
   void sample(snap *p = 0) {
     _sample(&last);
     if (p)
