@@ -15,13 +15,11 @@
 #ifndef PRIORITY_QUEUE_H
 #define PRIORITY_QUEUE_H
 
-#include "common/Mutex.h"
 #include "common/Formatter.h"
+#include "common/OpQueue.h"
 
 #include <map>
-#include <utility>
 #include <list>
-#include <algorithm>
 
 /**
  * Manages queue for normal and strict priority items
@@ -44,15 +42,15 @@
  * to provide fairness for different clients.
  */
 template <typename T, typename K>
-class PrioritizedQueue {
+class PrioritizedQueue : public OpQueue <T, K> {
   int64_t total_priority;
   int64_t max_tokens_per_subqueue;
   int64_t min_cost;
 
   typedef std::list<std::pair<unsigned, T> > ListPairs;
-  template <class F>
   static unsigned filter_list_pairs(
-    ListPairs *l, F f,
+    ListPairs *l,
+    std::function<bool (T)> f,
     std::list<T> *out) {
     unsigned ret = 0;
     if (out) {
@@ -106,14 +104,16 @@ class PrioritizedQueue {
     }
     void put_tokens(unsigned t) {
       tokens += t;
-      if (tokens > max_tokens)
+      if (tokens > max_tokens) {
 	tokens = max_tokens;
+      }
     }
     void take_tokens(unsigned t) {
-      if (tokens > t)
+      if (tokens > t) {
 	tokens -= t;
-      else
+      } else {
 	tokens = 0;
+      }
     }
     void enqueue(K cl, unsigned cost, T item) {
       q[cl].push_back(std::make_pair(cost, item));
@@ -136,12 +136,14 @@ class PrioritizedQueue {
       assert(!(q.empty()));
       assert(cur != q.end());
       cur->second.pop_front();
-      if (cur->second.empty())
+      if (cur->second.empty()) {
 	q.erase(cur++);
-      else
+      } else {
 	++cur;
-      if (cur == q.end())
+      }
+      if (cur == q.end()) {
 	cur = q.begin();
+      }
       size--;
     }
     unsigned length() const {
@@ -151,15 +153,17 @@ class PrioritizedQueue {
     bool empty() const {
       return q.empty();
     }
-    template <class F>
-    void remove_by_filter(F f, std::list<T> *out) {
+    void remove_by_filter(
+	std::function<bool (T)> f,
+       	std::list<T> *out) {
       for (typename Classes::iterator i = q.begin();
 	   i != q.end();
 	   ) {
 	size -= filter_list_pairs(&(i->second), f, out);
 	if (i->second.empty()) {
-	  if (cur == i)
+	  if (cur == i) {
 	    ++cur;
+	  }
 	  q.erase(i++);
 	} else {
 	  ++i;
@@ -170,11 +174,13 @@ class PrioritizedQueue {
     }
     void remove_by_class(K k, std::list<T> *out) {
       typename Classes::iterator i = q.find(k);
-      if (i == q.end())
+      if (i == q.end()) {
 	return;
+      }
       size -= i->second.size();
-      if (i == cur)
+      if (i == cur) {
 	++cur;
+      }
       if (out) {
 	for (typename ListPairs::reverse_iterator j =
 	       i->second.rbegin();
@@ -184,17 +190,19 @@ class PrioritizedQueue {
 	}
       }
       q.erase(i);
-      if (cur == q.end())
+      if (cur == q.end()) {
 	cur = q.begin();
+      }
     }
 
-    void dump(Formatter *f) const {
+    void dump(ceph::Formatter *f) const {
       f->dump_int("tokens", tokens);
       f->dump_int("max_tokens", max_tokens);
       f->dump_int("size", size);
       f->dump_int("num_keys", q.size());
-      if (!empty())
+      if (!empty()) {
 	f->dump_int("first_item_cost", front().first);
+      }
     }
   };
 
@@ -204,8 +212,9 @@ class PrioritizedQueue {
 
   SubQueue *create_queue(unsigned priority) {
     typename SubQueues::iterator p = queue.find(priority);
-    if (p != queue.end())
+    if (p != queue.end()) {
       return &p->second;
+    }
     total_priority += priority;
     SubQueue *sq = &queue[priority];
     sq->set_max_tokens(max_tokens_per_subqueue);
@@ -220,8 +229,9 @@ class PrioritizedQueue {
   }
 
   void distribute_tokens(unsigned cost) {
-    if (total_priority == 0)
+    if (total_priority == 0) {
       return;
+    }
     for (typename SubQueues::iterator i = queue.begin();
 	 i != queue.end();
 	 ++i) {
@@ -236,7 +246,7 @@ public:
       min_cost(min_c)
   {}
 
-  unsigned length() const {
+  unsigned length() const override final {
     unsigned total = 0;
     for (typename SubQueues::const_iterator i = queue.begin();
 	 i != queue.end();
@@ -253,8 +263,9 @@ public:
     return total;
   }
 
-  template <class F>
-  void remove_by_filter(F f, std::list<T> *removed = 0) {
+  void remove_by_filter(
+      std::function<bool (T)> f,
+      std::list<T> *removed = 0) override final {
     for (typename SubQueues::iterator i = queue.begin();
 	 i != queue.end();
 	 ) {
@@ -280,7 +291,7 @@ public:
     }
   }
 
-  void remove_by_class(K k, std::list<T> *out = 0) {
+  void remove_by_class(K k, std::list<T> *out = 0) override final {
     for (typename SubQueues::iterator i = queue.begin();
 	 i != queue.end();
 	 ) {
@@ -305,15 +316,15 @@ public:
     }
   }
 
-  void enqueue_strict(K cl, unsigned priority, T item) {
+  void enqueue_strict(K cl, unsigned priority, T item) override final {
     high_queue[priority].enqueue(cl, 0, item);
   }
 
-  void enqueue_strict_front(K cl, unsigned priority, T item) {
+  void enqueue_strict_front(K cl, unsigned priority, T item) override final {
     high_queue[priority].enqueue_front(cl, 0, item);
   }
 
-  void enqueue(K cl, unsigned priority, unsigned cost, T item) {
+  void enqueue(K cl, unsigned priority, unsigned cost, T item) override final {
     if (cost < min_cost)
       cost = min_cost;
     if (cost > max_tokens_per_subqueue)
@@ -321,7 +332,7 @@ public:
     create_queue(priority)->enqueue(cl, cost, item);
   }
 
-  void enqueue_front(K cl, unsigned priority, unsigned cost, T item) {
+  void enqueue_front(K cl, unsigned priority, unsigned cost, T item) override final {
     if (cost < min_cost)
       cost = min_cost;
     if (cost > max_tokens_per_subqueue)
@@ -329,20 +340,21 @@ public:
     create_queue(priority)->enqueue_front(cl, cost, item);
   }
 
-  bool empty() const {
+  bool empty() const override final {
     assert(total_priority >= 0);
     assert((total_priority == 0) || !(queue.empty()));
     return queue.empty() && high_queue.empty();
   }
 
-  T dequeue() {
+  T dequeue() override final {
     assert(!empty());
 
     if (!(high_queue.empty())) {
       T ret = high_queue.rbegin()->second.front().second;
       high_queue.rbegin()->second.pop_front();
-      if (high_queue.rbegin()->second.empty())
+      if (high_queue.rbegin()->second.empty()) {
 	high_queue.erase(high_queue.rbegin()->first);
+      }
       return ret;
     }
 
@@ -358,8 +370,9 @@ public:
 	unsigned cost = i->second.front().first;
 	i->second.take_tokens(cost);
 	i->second.pop_front();
-	if (i->second.empty())
+	if (i->second.empty()) {
 	  remove_queue(i->first);
+	}
 	distribute_tokens(cost);
 	return ret;
       }
@@ -370,13 +383,14 @@ public:
     T ret = queue.rbegin()->second.front().second;
     unsigned cost = queue.rbegin()->second.front().first;
     queue.rbegin()->second.pop_front();
-    if (queue.rbegin()->second.empty())
+    if (queue.rbegin()->second.empty()) {
       remove_queue(queue.rbegin()->first);
+    }
     distribute_tokens(cost);
     return ret;
   }
 
-  void dump(Formatter *f) const {
+  void dump(ceph::Formatter *f) const override final {
     f->dump_int("total_priority", total_priority);
     f->dump_int("max_tokens_per_subqueue", max_tokens_per_subqueue);
     f->dump_int("min_cost", min_cost);
