@@ -16,10 +16,10 @@
 #define CEPH_COMMON_CODE_ENVIRONMENT_H
 
 enum code_environment_t {
-  CODE_ENVIRONMENT_TEST = 0,
-  CODE_ENVIRONMENT_UTILITY = 1,
-  CODE_ENVIRONMENT_DAEMON = 2,
-  CODE_ENVIRONMENT_LIBRARY = 3,
+  CODE_ENVIRONMENT_UTILITY = 0,
+  CODE_ENVIRONMENT_DAEMON = 1,
+  CODE_ENVIRONMENT_LIBRARY = 2,
+  CODE_ENVIRONMENT_UTILITY_NODOUT = 3,
 };
 
 #ifdef __cplusplus
