@@ -18,11 +18,13 @@
 #include <boost/intrusive/list.hpp>
 #include <boost/intrusive/rbtree.hpp>
 #include <boost/intrusive/avltree.hpp>
+#include <boost/intrusive/slist.hpp>
 #include <mutex>
 #include <atomic>
 #include <vector>
 #include <algorithm>
 #include <iostream>
+
 #include "common/likely.h"
 
 #ifndef CACHE_LINE_SIZE
@@ -39,6 +41,7 @@ namespace cohort {
     /* public flag values */
     constexpr uint32_t FLAG_NONE = 0x0000;
     constexpr uint32_t FLAG_INITIAL = 0x0001;
+    constexpr uint32_t FLAG_RECYCLE = 0x0002;
 
     enum class Edge : std::uint8_t
     {
@@ -46,7 +49,7 @@ namespace cohort {
       LRU
     };
 
-    typedef bi::link_mode<bi::safe_link> link_mode; // for debugging
+    typedef bi::link_mode<bi::safe_link> link_mode;
 
     class Object
     {
@@ -62,6 +65,14 @@ namespace cohort {
 			 &Object::lru_hook >,
 		       bi::constant_time_size<true>> Queue;
 
+      bi::slist_member_hook< link_mode > q2_hook;
+
+      typedef bi::slist<Object,
+			bi::member_hook<
+			  Object, bi::slist_member_hook< link_mode >,
+			  &Object::q2_hook >,
+			bi::constant_time_size<true>> Queue2;
+
     public:
 
       Object() : lru_flags(FLAG_NONE), lru_refcnt(0), lru_adj(0) {}
@@ -75,6 +86,10 @@ namespace cohort {
     private:
       template <typename LK>
       friend class LRU;
+
+      template <typename T, typename TTree, typename CLT, typename CEQ,
+	      typename K, typename LK>
+      friend class TreeX;
     };
 
     /* allocator & recycler interface (create or re-use LRU objects) */
@@ -131,14 +146,9 @@ namespace cohort {
 	for (int ix = 0; ix < n_lanes; ++ix,
 	       lane_ix = next_evict_lane()) {
 	  Lane& lane = qlane[lane_ix];
+	  lane.lock.lock();
 	  /* if object at LRU has refcnt==1, it may be reclaimable */
 	  Object* o = &(lane.q.back());
-#if 0 /* XXX save for refactor */
-	  std::cout << __func__
-		    << " " << o
-		    << " refcnt: " << o->lru_refcnt
-		    << std::endl;
-#endif
 	  if (can_reclaim(o)) {
 	    ++(o->lru_refcnt);
 	    o->lru_flags |= FLAG_EVICTING;
@@ -157,7 +167,6 @@ namespace cohort {
 	      return o;
 	    } else {
 	      // XXX can't make unreachable (means what?)
-	      lane.lock.lock();
 	      --(o->lru_refcnt);
 	      o->lru_flags &= ~FLAG_EVICTING;
 	      /* unlock in next block */
@@ -198,6 +207,7 @@ namespace cohort {
 
       void unref(Object* o, uint32_t flags) {
 	uint32_t refcnt = --(o->lru_refcnt);
+	Object* tdo = nullptr;
 	if (unlikely(refcnt == 0)) {
 	  Lane& lane = lane_of(o);
 	  lane.lock.lock();
@@ -206,7 +216,7 @@ namespace cohort {
 	    Object::Queue::iterator it =
 	      Object::Queue::s_iterator_to(*o);
 	    lane.q.erase(it);
-	    delete o;
+	    tdo = o;
 	  }
 	  lane.lock.unlock();
 	} else if (unlikely(refcnt == SENTINEL_REFCNT)) {
@@ -220,21 +230,26 @@ namespace cohort {
 	    lane.q.erase(it);
 	    /* hiwat check */
 	    if (lane.q.size() > lane_hiwat) {
-	      delete o;
+	      tdo = o;
 	    } else {
 	      lane.q.push_back(*o);
 	    }
 	  }
 	  lane.lock.unlock();
 	}
+	/* unref out-of-line && !LOCKED */
+	if (tdo)
+	  delete tdo;
       } /* unref */
 
-      Object* insert(ObjectFactory* fac, Edge edge, uint32_t flags) {
+      Object* insert(ObjectFactory* fac, Edge edge, uint32_t& flags) {
 	/* use supplied functor to re-use an evicted object, or
 	 * allocate a new one of the descendant type */
 	Object* o = evict_block();
-	if (o)
+	if (o) {
 	  fac->recycle(o); /* recycle existing object */
+	  flags |= FLAG_RECYCLE;
+	}
 	else
 	  o = fac->alloc(); /* get a new one */
 
@@ -396,7 +411,7 @@ namespace cohort {
 	  v = lat.p->cache[slot];
 	  if (v) {
 	    if (CEQ()(*v, k)) {
-	      if (flags & (FLAG_LOCK|FLAG_UNLOCK))
+	      if ((flags & FLAG_LOCK) && (flags & FLAG_UNLOCK))
 		lat.lock->unlock();
 	      return v;
 	    }
@@ -414,7 +429,7 @@ namespace cohort {
 	    lat.p->cache[slot] = v;
 	  }
 	}
-	if (flags & (FLAG_LOCK|FLAG_UNLOCK))
+	if ((flags & FLAG_LOCK) && (flags & FLAG_UNLOCK))
 	  lat.lock->unlock();
 	return v;
       } /* find_latch */
@@ -456,6 +471,7 @@ namespace cohort {
 	/* clear a table, call supplied function on
 	 * each element found (e.g., retuns sentinel
 	 * references) */
+	Object::Queue2 drain_q;
 	for (int t_ix = 0; t_ix < n_part; ++t_ix) {
 	  Partition& p = part[t_ix];
 	  if (flags & FLAG_LOCK) /* LOCKED */
@@ -463,13 +479,19 @@ namespace cohort {
 	  while (p.tr.size() > 0) {
 	    iterator it = p.tr.begin();
 	    T* v = &(*it);
-	    p.tr.erase(it); /* must precede uref(v), in
-			     * safe_link mode */
-	    uref(v);
+	    p.tr.erase(it);
+	    drain_q.push_front(*v);
 	  }
 	  if (flags & FLAG_LOCK) /* we locked it, !LOCKED */
 	    p.lock.unlock();
 	} /* each partition */
+	/* unref out-of-line && !LOCKED */
+	while (drain_q.size() > 0) {
+	  Object::Queue2::iterator it = drain_q.begin();
+	  T* v = static_cast<T*>(&(*it));
+	  drain_q.erase(it); /* must precede uref(v) in safe_link mode */
+	  uref(v);
+	}
       } /* drain */
 
     private:
