@@ -1,5 +1,6 @@
 #include "include/ipaddr.h"
 
+#include <sys/socket.h>
 #include <arpa/inet.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,6 +34,9 @@ const struct sockaddr *find_ipv4_in_subnet(const struct ifaddrs *addrs,
     if (addrs->ifa_addr == NULL)
       continue;
 
+    if (strcmp(addrs->ifa_name, "lo") == 0)
+      continue;
+
     if (addrs->ifa_addr->sa_family != net->sin_family)
       continue;
 
@@ -74,13 +78,16 @@ const struct sockaddr *find_ipv6_in_subnet(const struct ifaddrs *addrs,
     if (addrs->ifa_addr == NULL)
       continue;
 
+    if (strcmp(addrs->ifa_name, "lo") == 0)
+      continue;
+
     if (addrs->ifa_addr->sa_family != net->sin6_family)
       continue;
 
     struct in6_addr *cur = &((struct sockaddr_in6*)addrs->ifa_addr)->sin6_addr;
     netmask_ipv6(cur, prefix_len, &temp);
 
-    if (memcmp(temp.s6_addr32, want.s6_addr32, sizeof(temp)) == 0)
+    if (IN6_ARE_ADDR_EQUAL(&temp, &want))
       return addrs->ifa_addr;
   }
 
@@ -103,7 +110,7 @@ const struct sockaddr *find_ip_in_subnet(const struct ifaddrs *addrs,
 }
 
 
-bool parse_network(const char *s, struct sockaddr *network, unsigned int *prefix_len) {
+bool parse_network(const char *s, struct sockaddr_storage *network, unsigned int *prefix_len) {
   char *slash = strchr((char*)s, '/');
   if (!slash) {
     // no slash
@@ -137,14 +144,14 @@ bool parse_network(const char *s, struct sockaddr *network, unsigned int *prefix
   int ok;
   ok = inet_pton(AF_INET, addr, &((struct sockaddr_in*)network)->sin_addr);
   if (ok) {
-    network->sa_family = AF_INET;
+    network->ss_family = AF_INET;
     return true;
   }
 
   // try parsing as ipv6
   ok = inet_pton(AF_INET6, addr, &((struct sockaddr_in6*)network)->sin6_addr);
   if (ok) {
-    network->sa_family = AF_INET6;
+    network->ss_family = AF_INET6;
     return true;
   }
 
