@@ -20,6 +20,10 @@
 #include "common/Mutex.h"
 #include "include/utime.h"
 
+#include "common/config_obs.h"
+#include "common/Mutex.h"
+#include "common/ceph_time.h"
+
 #include <stdint.h>
 #include <string>
 #include <vector>
@@ -91,6 +95,7 @@ public:
 
   void tset(int idx, utime_t v);
   void tinc(int idx, utime_t v);
+  void tinc(int idx, ceph::timespan v);
   utime_t tget(int idx) const;
 
   void reset();
