@@ -31,21 +31,29 @@ float strict_strtof(const char *str, std::string *err);
 
 uint64_t strict_sistrtoll(const char *str, std::string *err);
 
-template <typename Target>
-Target strict_si_cast(const char *str, std::string *err) {
-  uint64_t ret = strict_sistrtoll(str, err);
-  if (!err->empty())
-    return ret;
-  if (ret > (uint64_t)std::numeric_limits<Target>::max()) {
-    err->append("The option value '");
-    err->append(str);
-    err->append("' seems to be too large");
-    return 0;
+template<typename T>
+T strict_si_cast(const char *str, std::string *err);
+
+/* On enter buf points to the end of the buffer, e.g. where the least
+ * significant digit of the input number will be printed. Returns pointer to
+ * where the most significant digit were printed, including zero padding.
+ * Does NOT add zero at the end of buffer, this is responsibility of the caller.
+ */
+template<typename T, const unsigned base = 10, const unsigned width = 1>
+static inline
+char* ritoa(T u, char *buf)
+{
+  static_assert(std::is_unsigned<T>::value, "signed types are not supported");
+  static_assert(base <= 16, "extend character map below to support higher bases");
+  unsigned digits = 0;
+  while (u) {
+    *--buf = "0123456789abcdef"[u % base];
+    u /= base;
+    digits++;
   }
-  return ret;
+  while (digits++ < width)
+    *--buf = '0';
+  return buf;
 }
 
-template <>
-uint64_t strict_si_cast(const char *str, std::string *err);
-
 #endif
