@@ -8,7 +8,7 @@
 
 #define dout_subsys ceph_subsys_journaler
 #undef dout_prefix
-#define dout_prefix *_dout << "JournalPlayer: "
+#define dout_prefix *_dout << "JournalPlayer: " << this << " "
 
 namespace journal {
 
@@ -53,8 +53,7 @@ JournalPlayer::JournalPlayer(librados::IoCtx &ioctx,
   : m_cct(NULL), m_object_oid_prefix(object_oid_prefix),
     m_journal_metadata(journal_metadata), m_replay_handler(replay_handler),
     m_lock("JournalPlayer::m_lock"), m_state(STATE_INIT), m_splay_offset(0),
-    m_watch_enabled(false), m_watch_scheduled(false), m_watch_interval(0),
-    m_commit_object(0) {
+    m_watch_enabled(false), m_watch_scheduled(false), m_watch_interval(0) {
   m_replay_handler->get();
   m_ioctx.dup(ioctx);
   m_cct = reinterpret_cast<CephContext *>(m_ioctx.cct());
@@ -69,8 +68,9 @@ JournalPlayer::JournalPlayer(librados::IoCtx &ioctx,
     uint8_t splay_width = m_journal_metadata->get_splay_width();
     auto &active_position = commit_position.object_positions.front();
     m_active_tag_tid = active_position.tag_tid;
-    m_commit_object = active_position.object_number;
-    m_splay_offset = m_commit_object % splay_width;
+    m_commit_position_valid = true;
+    m_commit_position = active_position;
+    m_splay_offset = active_position.object_number % splay_width;
     for (auto &position : commit_position.object_positions) {
       uint8_t splay_offset = position.object_number % splay_width;
       m_commit_positions[splay_offset] = position;
@@ -94,6 +94,7 @@ void JournalPlayer::prefetch() {
   assert(m_state == STATE_INIT);
   m_state = STATE_PREFETCH;
 
+  m_active_set = m_journal_metadata->get_active_set();
   uint8_t splay_width = m_journal_metadata->get_splay_width();
   for (uint8_t splay_offset = 0; splay_offset < splay_width; ++splay_offset) {
     m_prefetch_splay_offsets.insert(splay_offset);
@@ -107,9 +108,7 @@ void JournalPlayer::prefetch() {
     splay_offset_to_objects[position.first] = position.second.object_number;
   }
 
-  // prefetch the active object for each splay offset (and the following object)
-  uint64_t active_set = m_journal_metadata->get_active_set();
-  uint64_t max_object_number = (splay_width * (active_set + 1)) - 1;
+  // prefetch the active object for each splay offset
   std::set<uint64_t> prefetch_object_numbers;
   for (uint8_t splay_offset = 0; splay_offset < splay_width; ++splay_offset) {
     uint64_t object_number = splay_offset;
@@ -118,9 +117,6 @@ void JournalPlayer::prefetch() {
     }
 
     prefetch_object_numbers.insert(object_number);
-    if (object_number + splay_width <= max_object_number) {
-      prefetch_object_numbers.insert(object_number + splay_width);
-    }
   }
 
   ldout(m_cct, 10) << __func__ << ": prefetching "
@@ -156,7 +152,7 @@ void JournalPlayer::shut_down(Context *on_finish) {
     ObjectPlayerPtr object_player = get_object_player();
     switch (m_watch_step) {
     case WATCH_STEP_FETCH_FIRST:
-      object_player = m_object_players.begin()->second.begin()->second;
+      object_player = m_object_players.begin()->second;
       // fallthrough
     case WATCH_STEP_FETCH_CURRENT:
       object_player->unwatch();
@@ -182,12 +178,7 @@ bool JournalPlayer::try_pop_front(Entry *entry, uint64_t *commit_tid) {
     if (!is_object_set_ready()) {
       m_handler_notified = false;
     } else {
-      if (!m_watch_enabled) {
-        notify_complete(0);
-      } else if (!m_watch_scheduled) {
-        m_handler_notified = false;
-        schedule_watch();
-      }
+      refetch(true);
     }
     return false;
   }
@@ -266,14 +257,10 @@ int JournalPlayer::process_prefetch(uint64_t object_number) {
 
   bool prefetch_complete = false;
   assert(m_object_players.count(splay_offset) == 1);
-  ObjectPlayers &object_players = m_object_players[splay_offset];
+  ObjectPlayerPtr object_player = m_object_players[splay_offset];
 
   // prefetch in-order since a newer splay object could prefetch first
-  while (m_fetch_object_numbers.count(
-           object_players.begin()->second->get_object_number()) == 0) {
-    ObjectPlayerPtr object_player = object_players.begin()->second;
-    uint64_t player_object_number = object_player->get_object_number();
-
+  if (m_fetch_object_numbers.count(object_player->get_object_number()) == 0) {
     // skip past known committed records
     if (m_commit_positions.count(splay_offset) != 0 &&
         !object_player->empty()) {
@@ -302,20 +289,6 @@ int JournalPlayer::process_prefetch(uint64_t object_number) {
         object_player->pop_front();
       }
 
-      // if this object contains the commit position, our read should start with
-      // the next consistent journal entry in the sequence
-      if (player_object_number == m_commit_object) {
-        if (object_player->empty()) {
-          advance_splay_object();
-        } else {
-          Entry entry;
-          object_player->front(&entry);
-          if (entry.get_tag_tid() == position.tag_tid) {
-            advance_splay_object();
-          }
-        }
-      }
-
       // do not search for commit position for this object
       // if we've already seen it
       if (found_commit) {
@@ -324,9 +297,14 @@ int JournalPlayer::process_prefetch(uint64_t object_number) {
     }
 
     // if the object is empty, pre-fetch the next splay object
-    if (!remove_empty_object_player(object_player)) {
+    if (object_player->empty() && object_player->refetch_required()) {
+      ldout(m_cct, 10) << "refetching potentially partially decoded object"
+                       << dendl;
+      object_player->set_refetch_state(ObjectPlayer::REFETCH_STATE_NONE);
+      fetch(object_player);
+    } else if (!remove_empty_object_player(object_player)) {
+      ldout(m_cct, 10) << "prefetch of object complete" << dendl;
       prefetch_complete = true;
-      break;
     }
   }
 
@@ -339,17 +317,32 @@ int JournalPlayer::process_prefetch(uint64_t object_number) {
     return 0;
   }
 
+  ldout(m_cct, 10) << "switching to playback mode" << dendl;
   m_state = STATE_PLAYBACK;
+
+  // if we have a valid commit position, our read should start with
+  // the next consistent journal entry in the sequence
+  if (m_commit_position_valid) {
+    splay_offset = m_commit_position.object_number % splay_width;
+    object_player = m_object_players[splay_offset];
+
+    if (object_player->empty()) {
+      if (!object_player->refetch_required()) {
+        advance_splay_object();
+      }
+    } else {
+      Entry entry;
+      object_player->front(&entry);
+      if (entry.get_tag_tid() == m_commit_position.tag_tid) {
+        advance_splay_object();
+      }
+    }
+  }
+
   if (verify_playback_ready()) {
     notify_entries_available();
   } else if (is_object_set_ready()) {
-    if (m_watch_enabled) {
-      schedule_watch();
-    } else {
-      ldout(m_cct, 10) << __func__ << ": no uncommitted entries available"
-                       << dendl;
-      notify_complete(0);
-    }
+    refetch(false);
   }
   return 0;
 }
@@ -361,17 +354,7 @@ int JournalPlayer::process_playback(uint64_t object_number) {
   if (verify_playback_ready()) {
     notify_entries_available();
   } else if (is_object_set_ready()) {
-    if (m_watch_enabled) {
-      schedule_watch();
-    } else {
-      ObjectPlayerPtr object_player = get_object_player();
-      uint8_t splay_width = m_journal_metadata->get_splay_width();
-      uint64_t active_set = m_journal_metadata->get_active_set();
-      uint64_t object_set = object_player->get_object_number() / splay_width;
-      if (object_set == active_set) {
-        notify_complete(0);
-      }
-    }
+    refetch(false);
   }
   return 0;
 }
@@ -379,8 +362,10 @@ int JournalPlayer::process_playback(uint64_t object_number) {
 bool JournalPlayer::is_object_set_ready() const {
   assert(m_lock.is_locked());
   if (m_watch_scheduled || !m_fetch_object_numbers.empty()) {
+    ldout(m_cct, 20) << __func__ << ": waiting for in-flight fetch" << dendl;
     return false;
   }
+
   return true;
 }
 
@@ -455,13 +440,6 @@ bool JournalPlayer::verify_playback_ready() {
       } else if (m_prune_tag_tid && *m_prune_tag_tid == *m_active_tag_tid) {
         ldout(m_cct, 10) << __func__ << ": no more entries" << dendl;
         return false;
-      } else if (!m_watch_enabled) {
-        // current playback position is empty so this tag is done
-        ldout(m_cct, 10) << __func__ << ": no more in-sequence entries: "
-                         << "object_num=" << object_num << ", "
-                         << "active_tag=" << *m_active_tag_tid << dendl;
-        prune_active_tag(boost::none);
-        continue;
       } else if (m_watch_enabled && m_watch_prune_active_tag) {
         // detected current tag is now longer active and we have re-read the
         // current object but it's still empty, so this tag is done
@@ -470,12 +448,24 @@ bool JournalPlayer::verify_playback_ready() {
                          << "active_tag " << *m_active_tag_tid << dendl;
         prune_active_tag(boost::none);
         continue;
-      } else if (m_watch_enabled && object_player->refetch_required()) {
+      } else if (object_player->refetch_required()) {
         // if the active object requires a refetch, don't proceed looking for a
         // new tag before this process completes
         ldout(m_cct, 10) << __func__ << ": refetch required: "
                          << "object_num=" << object_num << dendl;
         return false;
+      } else if (!m_watch_enabled) {
+        // current playback position is empty so this tag is done
+        ldout(m_cct, 10) << __func__ << ": no more in-sequence entries: "
+                         << "object_num=" << object_num << ", "
+                         << "active_tag=" << *m_active_tag_tid << dendl;
+        prune_active_tag(boost::none);
+        continue;
+      } else if (!m_watch_scheduled) {
+        // no more entries and we don't have an active watch in-progress
+        ldout(m_cct, 10) << __func__ << ": no more entries -- watch required"
+                         << dendl;
+        return false;
       }
     }
   }
@@ -492,29 +482,38 @@ void JournalPlayer::prune_tag(uint64_t tag_tid) {
     m_prune_tag_tid = tag_tid;
   }
 
-  for (auto &players : m_object_players) {
-    for (auto player_pair : players.second) {
-      ObjectPlayerPtr object_player = player_pair.second;
-      ldout(m_cct, 15) << __func__ << ": checking " << object_player->get_oid()
-                       << dendl;
-      while (!object_player->empty()) {
-        Entry entry;
-        object_player->front(&entry);
-        if (entry.get_tag_tid() == tag_tid) {
-          ldout(m_cct, 20) << __func__ << ": pruned " << entry << dendl;
-          object_player->pop_front();
-        } else {
-          break;
-        }
+  bool pruned = false;
+  for (auto &player_pair : m_object_players) {
+    ObjectPlayerPtr object_player(player_pair.second);
+    ldout(m_cct, 15) << __func__ << ": checking " << object_player->get_oid()
+                     << dendl;
+    while (!object_player->empty()) {
+      Entry entry;
+      object_player->front(&entry);
+      if (entry.get_tag_tid() == tag_tid) {
+        ldout(m_cct, 20) << __func__ << ": pruned " << entry << dendl;
+        object_player->pop_front();
+        pruned = true;
+      } else {
+        break;
       }
     }
+  }
 
-    // trim any empty players to prefetch the next available object
-    ObjectPlayers object_players(players.second);
-    for (auto player_pair : object_players) {
-      remove_empty_object_player(player_pair.second);
+  // avoid watch delay when pruning stale tags from journal objects
+  if (pruned) {
+    ldout(m_cct, 15) << __func__ << ": reseting refetch state to immediate"
+                     << dendl;
+    for (auto &player_pair : m_object_players) {
+      ObjectPlayerPtr object_player(player_pair.second);
+      object_player->set_refetch_state(ObjectPlayer::REFETCH_STATE_IMMEDIATE);
     }
   }
+
+  // trim empty player to prefetch the next available object
+  for (auto &player_pair : m_object_players) {
+    remove_empty_object_player(player_pair.second);
+  }
 }
 
 void JournalPlayer::prune_active_tag(const boost::optional<uint64_t>& tag_tid) {
@@ -531,23 +530,15 @@ void JournalPlayer::prune_active_tag(const boost::optional<uint64_t>& tag_tid) {
   prune_tag(active_tag_tid);
 }
 
-const JournalPlayer::ObjectPlayers &JournalPlayer::get_object_players() const {
+ObjectPlayerPtr JournalPlayer::get_object_player() const {
   assert(m_lock.is_locked());
 
   SplayedObjectPlayers::const_iterator it = m_object_players.find(
     m_splay_offset);
   assert(it != m_object_players.end());
-
   return it->second;
 }
 
-ObjectPlayerPtr JournalPlayer::get_object_player() const {
-  assert(m_lock.is_locked());
-
-  const ObjectPlayers &object_players = get_object_players();
-  return object_players.begin()->second;
-}
-
 ObjectPlayerPtr JournalPlayer::get_object_player(uint64_t object_number) const {
   assert(m_lock.is_locked());
 
@@ -556,17 +547,9 @@ ObjectPlayerPtr JournalPlayer::get_object_player(uint64_t object_number) const {
   auto splay_it = m_object_players.find(splay_offset);
   assert(splay_it != m_object_players.end());
 
-  const ObjectPlayers &object_players = splay_it->second;
-  auto player_it = object_players.find(object_number);
-  assert(player_it != object_players.end());
-  return player_it->second;
-}
-
-ObjectPlayerPtr JournalPlayer::get_next_set_object_player() const {
-  assert(m_lock.is_locked());
-
-  const ObjectPlayers &object_players = get_object_players();
-  return object_players.rbegin()->second;
+  ObjectPlayerPtr object_player = splay_it->second;
+  assert(object_player->get_object_number() == object_number);
+  return object_player;
 }
 
 void JournalPlayer::advance_splay_object() {
@@ -587,10 +570,18 @@ bool JournalPlayer::remove_empty_object_player(const ObjectPlayerPtr &player) {
   uint64_t active_set = m_journal_metadata->get_active_set();
   if (!player->empty() || object_set == active_set) {
     return false;
-  } else if (m_watch_enabled && player->refetch_required()) {
+  } else if (player->refetch_required()) {
     ldout(m_cct, 20) << __func__ << ": " << player->get_oid() << " requires "
                      << "a refetch" << dendl;
     return false;
+  } else if (m_active_set != active_set) {
+    ldout(m_cct, 20) << __func__ << ": new active set detected, all players "
+                     << "require refetch" << dendl;
+    m_active_set = active_set;
+    for (auto &pair : m_object_players) {
+      pair.second->set_refetch_state(ObjectPlayer::REFETCH_STATE_IMMEDIATE);
+    }
+    return false;
   }
 
   ldout(m_cct, 15) << __func__ << ": " << player->get_oid() << " empty"
@@ -599,35 +590,35 @@ bool JournalPlayer::remove_empty_object_player(const ObjectPlayerPtr &player) {
   m_watch_prune_active_tag = false;
   m_watch_step = WATCH_STEP_FETCH_CURRENT;
 
-  ObjectPlayers &object_players = m_object_players[
-    player->get_object_number() % splay_width];
-  assert(!object_players.empty());
-
-  uint64_t next_object_num = object_players.rbegin()->first + splay_width;
-  uint64_t next_object_set = next_object_num / splay_width;
-  if (next_object_set <= active_set) {
-    fetch(next_object_num);
-  }
-  object_players.erase(player->get_object_number());
+  uint64_t next_object_num = player->get_object_number() + splay_width;
+  fetch(next_object_num);
   return true;
 }
 
 void JournalPlayer::fetch(uint64_t object_num) {
   assert(m_lock.is_locked());
 
-  std::string oid = utils::get_object_name(m_object_oid_prefix, object_num);
+  ObjectPlayerPtr object_player(new ObjectPlayer(
+    m_ioctx, m_object_oid_prefix, object_num, m_journal_metadata->get_timer(),
+    m_journal_metadata->get_timer_lock(), m_journal_metadata->get_order(),
+    m_journal_metadata->get_settings().max_fetch_bytes));
 
+  uint8_t splay_width = m_journal_metadata->get_splay_width();
+  m_object_players[object_num % splay_width] = object_player;
+  fetch(object_player);
+}
+
+void JournalPlayer::fetch(const ObjectPlayerPtr &object_player) {
+  assert(m_lock.is_locked());
+
+  uint64_t object_num = object_player->get_object_number();
+  std::string oid = utils::get_object_name(m_object_oid_prefix, object_num);
   assert(m_fetch_object_numbers.count(object_num) == 0);
   m_fetch_object_numbers.insert(object_num);
 
   ldout(m_cct, 10) << __func__ << ": " << oid << dendl;
   C_Fetch *fetch_ctx = new C_Fetch(this, object_num);
-  ObjectPlayerPtr object_player(new ObjectPlayer(
-    m_ioctx, m_object_oid_prefix, object_num, m_journal_metadata->get_timer(),
-    m_journal_metadata->get_timer_lock(), m_journal_metadata->get_order()));
 
-  uint8_t splay_width = m_journal_metadata->get_splay_width();
-  m_object_players[object_num % splay_width][object_num] = object_player;
   object_player->fetch(fetch_ctx);
 }
 
@@ -644,9 +635,6 @@ void JournalPlayer::handle_fetched(uint64_t object_num, int r) {
     return;
   }
 
-  if (r == -ENOENT) {
-    r = 0;
-  }
   if (r == 0) {
     ObjectPlayerPtr object_player = get_object_player(object_num);
     remove_empty_object_player(object_player);
@@ -654,7 +642,28 @@ void JournalPlayer::handle_fetched(uint64_t object_num, int r) {
   process_state(object_num, r);
 }
 
-void JournalPlayer::schedule_watch() {
+void JournalPlayer::refetch(bool immediate) {
+  ldout(m_cct, 10) << __func__ << dendl;
+  assert(m_lock.is_locked());
+  m_handler_notified = false;
+
+  // if watching the object, handle the periodic re-fetch
+  if (m_watch_enabled) {
+    schedule_watch(immediate);
+    return;
+  }
+
+  ObjectPlayerPtr object_player = get_object_player();
+  if (object_player->refetch_required()) {
+    object_player->set_refetch_state(ObjectPlayer::REFETCH_STATE_NONE);
+    fetch(object_player);
+    return;
+  }
+
+  notify_complete(0);
+}
+
+void JournalPlayer::schedule_watch(bool immediate) {
   ldout(m_cct, 10) << __func__ << dendl;
   assert(m_lock.is_locked());
   if (m_watch_scheduled) {
@@ -688,17 +697,20 @@ void JournalPlayer::schedule_watch() {
       uint8_t splay_width = m_journal_metadata->get_splay_width();
       uint64_t active_set = m_journal_metadata->get_active_set();
       uint64_t object_set = object_player->get_object_number() / splay_width;
-      if (object_set < active_set && object_player->refetch_required()) {
-        ldout(m_cct, 20) << __func__ << ": refetching "
+      if (immediate ||
+          (object_player->get_refetch_state() ==
+             ObjectPlayer::REFETCH_STATE_IMMEDIATE) ||
+          (object_set < active_set && object_player->refetch_required())) {
+        ldout(m_cct, 20) << __func__ << ": immediately refetching "
                          << object_player->get_oid()
                          << dendl;
-        object_player->clear_refetch_required();
+        object_player->set_refetch_state(ObjectPlayer::REFETCH_STATE_NONE);
         watch_interval = 0;
       }
     }
     break;
   case WATCH_STEP_FETCH_FIRST:
-    object_player = m_object_players.begin()->second.begin()->second;
+    object_player = m_object_players.begin()->second;
     watch_interval = 0;
     break;
   default:
@@ -762,7 +774,7 @@ void JournalPlayer::handle_watch_assert_active(int r) {
 
   m_watch_step = WATCH_STEP_FETCH_CURRENT;
   if (!m_shut_down && m_watch_enabled) {
-    schedule_watch();
+    schedule_watch(false);
   }
   m_async_op_tracker.finish_op();
 }
