@@ -12,6 +12,6 @@ void calc_snap_set_diff(CephContext *cct,
 			const librados::snap_set_t& snap_set,
 			librados::snap_t start, librados::snap_t end,
 			interval_set<uint64_t> *diff, uint64_t *end_size,
-			bool *end_exists);
+			bool *end_exists, librados::snap_t *clone_end_snap_id);
 
 #endif
