@@ -5,9 +5,6 @@
 
 #include "include/int_types.h"
 
-#include "include/rados/librados.hpp"
-
-#include "cls/rbd/cls_rbd_client.h"
 #include "librbd/parent_types.h"
 
 namespace librbd {
