@@ -4,12 +4,12 @@
 #ifndef CEPH_LIBRBD_OBJECT_MAP_LOCK_REQUEST_H
 #define CEPH_LIBRBD_OBJECT_MAP_LOCK_REQUEST_H
 
-#include "include/int_types.h"
 #include "include/buffer.h"
-#include "include/Context.h"
 #include "cls/lock/cls_lock_types.h"
 #include <map>
 
+class Context;
+
 namespace librbd {
 
 class ImageCtx;
