@@ -5,7 +5,6 @@
 #define CEPH_LIBRBD_RENAME_REQUEST_H
 
 #include "librbd/operation/Request.h"
-#include <iosfwd>
 #include <string>
 
 class Context;
@@ -58,6 +57,7 @@ public:
 protected:
   virtual void send_op();
   virtual bool should_complete(int r);
+  int filter_return_code(int r) const override;
 
   virtual journal::Event create_event(uint64_t op_tid) const {
     return journal::RenameEvent(op_tid, m_dest_name);
@@ -73,8 +73,6 @@ private:
 
   bufferlist m_header_bl;
 
-  int filter_state_return_code(int r);
-
   void send_read_source_header();
   void send_write_destination_header();
   void send_update_directory();
