@@ -204,10 +204,10 @@ void CDentry::mark_new()
   state_set(STATE_NEW);
 }
 
-void CDentry::make_path_string(string& s) const
+void CDentry::make_path_string(string& s, bool projected) const
 {
   if (dir) {
-    dir->inode->make_path_string(s);
+    dir->inode->make_path_string(s, projected);
   } else {
     s = "???";
   }
@@ -215,35 +215,13 @@ void CDentry::make_path_string(string& s) const
   s.append(name.data(), name.length());
 }
 
-void CDentry::make_path(filepath& fp) const
+void CDentry::make_path(filepath& fp, bool projected) const
 {
   assert(dir);
-  if (dir->inode->is_base())
-    fp = filepath(dir->inode->ino());               // base case
-  else if (dir->inode->get_parent_dn())
-    dir->inode->get_parent_dn()->make_path(fp);  // recurse
-  else
-    fp = filepath(dir->inode->ino());               // relative but not base?  hrm!
+  dir->inode->make_path(fp, projected);
   fp.push_dentry(name);
 }
 
-/*
-void CDentry::make_path(string& s, inodeno_t tobase)
-{
-  assert(dir);
-  
-  if (dir->inode->is_root()) {
-    s += "/";  // make it an absolute path (no matter what) if we hit the root.
-  } 
-  else if (dir->inode->get_parent_dn() &&
-	   dir->inode->ino() != tobase) {
-    dir->inode->get_parent_dn()->make_path(s, tobase);
-    s += "/";
-  }
-  s += name;
-}
-*/
-
 /*
  * we only add ourselves to remote_parents when the linkage is
  * active (no longer projected).  if the passed dnl is projected,
@@ -621,57 +599,3 @@ std::string CDentry::linkage_t::get_remote_d_type_string() const
     default: assert(0); return "";
   }
 }
-
-void CDentry::scrub_initialize(CDir *parent, bool recurse, bool children,
-                        ScrubHeaderRefConst header,
-                               Context *f)
-{
-  if (!scrub_infop)
-    scrub_info_create();
-  else
-    assert(!scrub_infop->dentry_scrubbing);
-
-  scrub_infop->scrub_parent = parent;
-  scrub_infop->scrub_recursive = recurse;
-  scrub_infop->scrub_children = children;
-  scrub_infop->dentry_scrubbing = true;
-  scrub_infop->on_finish = f;
-  scrub_infop->header = header;
-
-  auth_pin(this);
-}
-
-void CDentry::scrub_finished(Context **c)
-{
-  dout(10) << __func__ << dendl;
-  assert(scrub_info()->dentry_scrubbing);
-
-  if (scrub_infop->scrub_parent) {
-    scrub_infop->scrub_parent->scrub_dentry_finished(this);
-  }
-
-  *c = scrub_infop->on_finish;
-
-  if (scrub_infop->header && scrub_infop->header->origin == this) {
-    // We are at the point that a tagging scrub was initiated
-    LogChannelRef clog = dir->cache->mds->clog;
-    clog->info() << "scrub complete with tag '"
-      << scrub_infop->header->tag << "'";
-  }
-
-  delete scrub_infop;
-  scrub_infop = NULL;
-
-  auth_unpin(this);
-}
-
-void CDentry::scrub_info_create() const
-{
-  assert(!scrub_infop);
-
-  // break out of const-land to set up implicit initial state
-  CDentry *me = const_cast<CDentry*>(this);
-
-  // we don't need to change or set up any default parameters; assign directly
-  me->scrub_infop = new scrub_info_t();
-}
