@@ -41,21 +41,19 @@ void SnapServer::reset_state()
   // find any removed snapshot in data pools
   if (mds) {  // only if I'm running in a live MDS
     snapid_t first_free = 0;
-    const OSDMap *osdmap = mds->objecter->get_osdmap_read();
-    for (set<int64_t>::const_iterator p = mds->mdsmap->get_data_pools().begin();
-         p != mds->mdsmap->get_data_pools().end();
-         ++p) {
-      const pg_pool_t *pi = osdmap->get_pg_pool(*p);
-      if (!pi) {
-        // If pool isn't in OSDMap yet then can't have any snaps needing
-        // removal, skip.
-        continue;
-      }
-      if (!pi->removed_snaps.empty() &&
-          pi->removed_snaps.range_end() > first_free)
-        first_free = pi->removed_snaps.range_end();
-    }
-    mds->objecter->put_osdmap_read();
+    mds->objecter->with_osdmap([&](const OSDMap& o) {
+	for (const auto p : mds->mdsmap->get_data_pools()) {
+	  const pg_pool_t *pi = o.get_pg_pool(p);
+	  if (!pi) {
+	    // If pool isn't in OSDMap yet then can't have any snaps
+	    // needing removal, skip.
+	    continue;
+	  }
+	  if (!pi->removed_snaps.empty() &&
+	      pi->removed_snaps.range_end() > first_free)
+	    first_free = pi->removed_snaps.range_end();
+	}
+      });
     if (first_free > last_snap)
       last_snap = first_free;
   }
@@ -259,30 +257,28 @@ void SnapServer::check_osd_map(bool force)
   map<int, vector<snapid_t> > all_purge;
   map<int, vector<snapid_t> > all_purged;
 
-  const OSDMap *osdmap = mds->objecter->get_osdmap_read();
-  for (map<int, set<snapid_t> >::iterator p = need_to_purge.begin();
-       p != need_to_purge.end();
-       ++p) {
-    int id = p->first;
-    const pg_pool_t *pi = osdmap->get_pg_pool(id);
-    if (pi == NULL) {
-      // The pool is gone.  So are the snapshots.
-      all_purged[id] = std::vector<snapid_t>(p->second.begin(), p->second.end());
-      continue;
-    }
+  mds->objecter->with_osdmap(
+    [this, &all_purged, &all_purge](const OSDMap& osdmap) {
+      for (const auto& p : need_to_purge) {
+	int id = p.first;
+	const pg_pool_t *pi = osdmap.get_pg_pool(id);
+	if (pi == NULL) {
+	  // The pool is gone.  So are the snapshots.
+	  all_purged[id] = std::vector<snapid_t>(p.second.begin(),
+						 p.second.end());
+	  continue;
+	}
 
-    for (set<snapid_t>::iterator q = p->second.begin();
-	 q != p->second.end();
-	 ++q) {
-      if (pi->is_removed_snap(*q)) {
-	dout(10) << " osdmap marks " << *q << " as removed" << dendl;
-	all_purged[id].push_back(*q);
-      } else {
-	all_purge[id].push_back(*q);
+	for (const auto& q : p.second) {
+	  if (pi->is_removed_snap(q)) {
+	    dout(10) << " osdmap marks " << q << " as removed" << dendl;
+	    all_purged[id].push_back(q);
+	  } else {
+	    all_purge[id].push_back(q);
+	  }
+	}
       }
-    }
-  }
-  mds->objecter->put_osdmap_read();
+  });
 
   if (!all_purged.empty()) {
     // prepare to remove from need_to_purge list
