@@ -18,7 +18,7 @@
 #include "common/config.h"
 #include "include/types.h"
 
-#include "../MDS.h"
+#include "../MDSRank.h"
 
 #include "EMetaBlob.h"
 #include "../LogEvent.h"
@@ -44,12 +44,13 @@ public:
 
   EMetaBlob *get_metablob() { return &metablob; }
 
-  void encode(bufferlist& bl) const;
+  void encode(bufferlist& bl, uint64_t features) const;
   void decode(bufferlist::iterator &bl);
   void dump(Formatter *f) const;
   static void generate_test_instances(list<EExport*>& ls);
-  void replay(MDS *mds);
+  void replay(MDSRank *mds);
 
 };
+WRITE_CLASS_ENCODER_FEATURES(EExport)
 
 #endif
