@@ -40,7 +40,7 @@ class EImportFinish : public LogEvent {
       out << " failed";
   }
 
-  void encode(bufferlist& bl) const;
+  void encode(bufferlist& bl, uint64_t features) const;
   void decode(bufferlist::iterator &bl);
   void dump(Formatter *f) const;
   static void generate_test_instances(list<EImportFinish*>& ls);
@@ -48,5 +48,6 @@ class EImportFinish : public LogEvent {
   void replay(MDSRank *mds);
 
 };
+WRITE_CLASS_ENCODER_FEATURES(EImportFinish)
 
 #endif
