@@ -47,7 +47,7 @@ protected:
 
   EMetaBlob *get_metablob() { return &metablob; }
   
-  void encode(bufferlist &bl) const;
+  void encode(bufferlist &bl, uint64_t features) const;
   void decode(bufferlist::iterator &bl);
   void dump(Formatter *f) const;
   static void generate_test_instances(list<EImportStart*>& ls);
@@ -56,5 +56,6 @@ protected:
   void replay(MDSRank *mds);
 
 };
+WRITE_CLASS_ENCODER_FEATURES(EImportStart)
 
 #endif
