@@ -10,20 +10,25 @@
 
 static multimap<ceph_filelock, ceph_lock_state_t*> global_waiting_locks;
 
+static void remove_global_waiting(ceph_filelock &fl, ceph_lock_state_t *lock_state)
+{
+  for (auto p = global_waiting_locks.find(fl);
+       p != global_waiting_locks.end(); ) {
+    if (p->first != fl)
+      break;
+    if (p->second == lock_state) {
+      global_waiting_locks.erase(p);
+      break;
+    }
+    ++p;
+  }
+}
+
 ceph_lock_state_t::~ceph_lock_state_t()
 {
   if (type == CEPH_LOCK_FCNTL) {
     for (auto p = waiting_locks.begin(); p != waiting_locks.end(); ++p) {
-      for (auto q = global_waiting_locks.find(p->second);
-	   q != global_waiting_locks.end(); ) {
-	if (q->first != p->second)
-	  break;
-	if (q->second == this) {
-	  global_waiting_locks.erase(q);
-	  break;
-	}
-	++q;
-      }
+      remove_global_waiting(p->second, this);
     }
   }
 }
@@ -50,6 +55,9 @@ void ceph_lock_state_t::remove_waiting(const ceph_filelock& fl)
       break;
     if (p->second.length == fl.length &&
 	ceph_filelock_owner_equal(p->second, fl)) {
+      if (type == CEPH_LOCK_FCNTL) {
+	remove_global_waiting(p->second, this);
+      }
       waiting_locks.erase(p);
       --client_waiting_lock_counts[(client_t)fl.client];
       if (!client_waiting_lock_counts[(client_t)fl.client]) {
@@ -59,19 +67,6 @@ void ceph_lock_state_t::remove_waiting(const ceph_filelock& fl)
     }
     ++p;
   }
-
-  if (type == CEPH_LOCK_FCNTL) {
-    for (auto q = global_waiting_locks.find(fl);
-	 q != global_waiting_locks.end(); ) {
-      if (q->first != fl)
-	break;
-      if (q->second == this) {
-	global_waiting_locks.erase(q);
-	break;
-      }
-      ++q;
-    }
-  }
 }
 
 bool ceph_lock_state_t::is_deadlock(const ceph_filelock& fl,
@@ -141,6 +136,7 @@ bool ceph_lock_state_t::is_deadlock(const ceph_filelock& fl,
 void ceph_lock_state_t::add_waiting(const ceph_filelock& fl)
 {
   waiting_locks.insert(pair<uint64_t, ceph_filelock>(fl.start, fl));
+  ++client_waiting_lock_counts[(client_t)fl.client];
   if (type == CEPH_LOCK_FCNTL) {
     global_waiting_locks.insert(pair<ceph_filelock,ceph_lock_state_t*>(fl, this));
   }
@@ -200,8 +196,6 @@ bool ceph_lock_state_t::add_lock(ceph_filelock& new_lock,
   if (ret) {
     ++client_held_lock_counts[(client_t)new_lock.client];
   }
-  else if (wait_on_fail && !replay)
-    ++client_waiting_lock_counts[(client_t)new_lock.client];
   return ret;
 }
 
@@ -320,16 +314,8 @@ bool ceph_lock_state_t::remove_all_from (client_t client)
 	++iter;
 	continue;
       }
-
-      for (auto p = global_waiting_locks.find(iter->second);
-	   p != global_waiting_locks.end(); ) {
-	if (p->first != iter->second)
-	  break;
-	if (p->second == this) {
-	  global_waiting_locks.erase(p);
-	  break;
-	}
-	++p;
+      if (type == CEPH_LOCK_FCNTL) {
+	remove_global_waiting(iter->second, this);
       }
       waiting_locks.erase(iter++);
     }
