@@ -19,12 +19,20 @@
 
 
 class MClientCapRelease : public Message {
+  static const int HEAD_VERSION = 2;
+  static const int COMPAT_VERSION = 1;
  public:
   struct ceph_mds_cap_release head;
   vector<ceph_mds_cap_item> caps;
 
+  // The message receiver must wait for this OSD epoch
+  // before actioning this cap release.
+  epoch_t osd_epoch_barrier;
+
   MClientCapRelease() : 
-    Message(CEPH_MSG_CLIENT_CAPRELEASE) {
+    Message(CEPH_MSG_CLIENT_CAPRELEASE, HEAD_VERSION, COMPAT_VERSION),
+    osd_epoch_barrier(0)
+  {
     memset(&head, 0, sizeof(head));
   }
 private:
@@ -33,18 +41,22 @@ private:
 public:
   const char *get_type_name() const { return "client_cap_release";}
   void print(ostream& out) const {
-    out << "client_cap_release(" << head.num << ")";
+    out << "client_cap_release(" << caps.size() << ")";
   }
   
   void decode_payload() {
     bufferlist::iterator p = payload.begin();
     ::decode(head, p);
     ::decode_nohead(head.num, caps, p);
+    if (header.version >= 2) {
+      ::decode(osd_epoch_barrier, p);
+    }
   }
   void encode_payload(uint64_t features) {
     head.num = caps.size();
     ::encode(head, payload);
     ::encode_nohead(caps, payload);
+    ::encode(osd_epoch_barrier, payload);
   }
 };
 
