@@ -20,12 +20,20 @@
 
 
 class MClientCaps : public Message {
-  static const int HEAD_VERSION = 7;
+  static const int HEAD_VERSION = 8;
   static const int COMPAT_VERSION = 1;
 
  public:
-  struct ceph_mds_caps head;
+  struct ceph_mds_caps_head head;
+
+  uint64_t size, max_size, truncate_size;
+  uint32_t truncate_seq;
+  utime_t mtime, atime, ctime;
+  file_layout_t layout;
+  uint32_t time_warp_seq;
+
   struct ceph_mds_cap_peer peer;
+
   bufferlist snapbl;
   bufferlist xattrbl;
   bufferlist flockbl;
@@ -49,16 +57,18 @@ class MClientCaps : public Message {
   inodeno_t get_realm() { return inodeno_t(head.realm); }
   uint64_t get_cap_id() { return head.cap_id; }
 
-  uint64_t get_size() { return head.size;  }
-  uint64_t get_max_size() { return head.max_size;  }
-  __u32 get_truncate_seq() { return head.truncate_seq; }
-  uint64_t get_truncate_size() { return head.truncate_size; }
-  utime_t get_ctime() { return utime_t(head.ctime); }
-  utime_t get_mtime() { return utime_t(head.mtime); }
-  utime_t get_atime() { return utime_t(head.atime); }
-  __u32 get_time_warp_seq() { return head.time_warp_seq; }
+  uint64_t get_size() { return size;  }
+  uint64_t get_max_size() { return max_size;  }
+  __u32 get_truncate_seq() { return truncate_seq; }
+  uint64_t get_truncate_size() { return truncate_size; }
+  utime_t get_ctime() { return ctime; }
+  utime_t get_mtime() { return mtime; }
+  utime_t get_atime() { return atime; }
+  __u32 get_time_warp_seq() { return time_warp_seq; }
 
-  ceph_file_layout& get_layout() { return head.layout; }
+  const file_layout_t& get_layout() {
+    return layout;
+  }
 
   int       get_migrate_seq() { return head.migrate_seq; }
   int       get_op() { return head.op; }
@@ -72,14 +82,15 @@ class MClientCaps : public Message {
   void set_caps(int c) { head.caps = c; }
   void set_wanted(int w) { head.wanted = w; }
 
-  void set_max_size(uint64_t ms) { head.max_size = ms; }
+  void set_max_size(uint64_t ms) { max_size = ms; }
 
   void set_migrate_seq(unsigned m) { head.migrate_seq = m; }
   void set_op(int o) { head.op = o; }
 
-  void set_size(loff_t s) { head.size = s; }
-  void set_mtime(const utime_t &t) { t.encode_timeval(&head.mtime); }
-  void set_atime(const utime_t &t) { t.encode_timeval(&head.atime); }
+  void set_size(loff_t s) { size = s; }
+  void set_mtime(const utime_t &t) { mtime = t; }
+  void set_ctime(const utime_t &t) { ctime = t; }
+  void set_atime(const utime_t &t) { atime = t; }
 
   void set_cap_peer(uint64_t id, ceph_seq_t seq, ceph_seq_t mseq, int mds, int flags) {
     peer.cap_id = id;
@@ -92,8 +103,15 @@ class MClientCaps : public Message {
   void set_oldest_flush_tid(ceph_tid_t tid) { oldest_flush_tid = tid; }
   ceph_tid_t get_oldest_flush_tid() { return oldest_flush_tid; }
 
+  void clear_dirty() { head.dirty = 0; }
+
   MClientCaps()
     : Message(CEPH_MSG_CLIENT_CAPS, HEAD_VERSION, COMPAT_VERSION),
+      size(0),
+      max_size(0),
+      truncate_size(0),
+      truncate_seq(0),
+      time_warp_seq(0),
       osd_epoch_barrier(0),
       oldest_flush_tid(0),
       caller_uid(0), caller_gid(0) {
@@ -110,6 +128,11 @@ class MClientCaps : public Message {
 	      int mseq,
               epoch_t oeb)
     : Message(CEPH_MSG_CLIENT_CAPS, HEAD_VERSION, COMPAT_VERSION),
+      size(0),
+      max_size(0),
+      truncate_size(0),
+      truncate_seq(0),
+      time_warp_seq(0),
       osd_epoch_barrier(oeb),
       oldest_flush_tid(0),
       caller_uid(0), caller_gid(0) {
@@ -130,6 +153,11 @@ class MClientCaps : public Message {
 	      inodeno_t ino, inodeno_t realm,
 	      uint64_t id, int mseq, epoch_t oeb)
     : Message(CEPH_MSG_CLIENT_CAPS, HEAD_VERSION, COMPAT_VERSION),
+      size(0),
+      max_size(0),
+      truncate_size(0),
+      truncate_seq(0),
+      time_warp_seq(0),
       osd_epoch_barrier(oeb),
       oldest_flush_tid(0),
       caller_uid(0), caller_gid(0) {
@@ -161,12 +189,12 @@ public:
     if (head.migrate_seq)
       out << " mseq " << head.migrate_seq;
 
-    out << " size " << head.size << "/" << head.max_size;
-    if (head.truncate_seq)
-      out << " ts " << head.truncate_seq;
-    out << " mtime " << utime_t(head.mtime);
-    if (head.time_warp_seq)
-      out << " tws " << head.time_warp_seq;
+    out << " size " << size << "/" << max_size;
+    if (truncate_seq)
+      out << " ts " << truncate_seq << "/" << truncate_size;
+    out << " mtime " << mtime;
+    if (time_warp_seq)
+      out << " tws " << time_warp_seq;
 
     if (head.xattr_version)
       out << " xattrs(v=" << head.xattr_version << " l=" << xattrbl.length() << ")";
@@ -177,6 +205,21 @@ public:
   void decode_payload() {
     bufferlist::iterator p = payload.begin();
     ::decode(head, p);
+    ceph_mds_caps_body_legacy body;
+    ::decode(body, p);
+    if (head.op == CEPH_CAP_OP_EXPORT) {
+      peer = body.peer;
+    } else {
+      size = body.size;
+      max_size = body.max_size;
+      truncate_size = body.truncate_size;
+      truncate_seq = body.truncate_seq;
+      mtime = utime_t(body.mtime);
+      atime = utime_t(body.atime);
+      ctime = utime_t(body.ctime);
+      layout.from_legacy(body.layout);
+      time_warp_seq = body.time_warp_seq;
+    }
     ::decode_nohead(head.snap_trace_len, snapbl, p);
 
     assert(middle.length() == head.xattr_len);
@@ -190,8 +233,6 @@ public:
     if (header.version >= 3) {
       if (head.op == CEPH_CAP_OP_IMPORT)
 	::decode(peer, p);
-      else if (head.op == CEPH_CAP_OP_EXPORT)
-	memcpy(&peer, &head.peer, sizeof(peer));
     }
 
     if (header.version >= 4) {
@@ -211,17 +252,31 @@ public:
       ::decode(caller_uid, p);
       ::decode(caller_gid, p);
     }
+    if (header.version >= 8) {
+      ::decode(layout.pool_ns, p);
+    }
   }
   void encode_payload(uint64_t features) {
     header.version = HEAD_VERSION;
     head.snap_trace_len = snapbl.length();
     head.xattr_len = xattrbl.length();
 
-    // record peer in unused fields of cap export message
-    if ((features & CEPH_FEATURE_EXPORT_PEER) && head.op == CEPH_CAP_OP_EXPORT)
-      memcpy(&head.peer, &peer, sizeof(peer));
-
     ::encode(head, payload);
+    ceph_mds_caps_body_legacy body;
+    if (head.op == CEPH_CAP_OP_EXPORT) {
+      body.peer = peer;
+    } else {
+      body.size = size;
+      body.max_size = max_size;
+      body.truncate_size = truncate_size;
+      body.truncate_seq = truncate_seq;
+      mtime.encode_timeval(&body.mtime);
+      atime.encode_timeval(&body.atime);
+      ctime.encode_timeval(&body.ctime);
+      layout.to_legacy(&body.layout);
+      body.time_warp_seq = time_warp_seq;
+    }
+    ::encode(body, payload);
     ::encode_nohead(snapbl, payload);
 
     middle = xattrbl;
@@ -254,6 +309,8 @@ public:
     ::encode(oldest_flush_tid, payload);
     ::encode(caller_uid, payload);
     ::encode(caller_gid, payload);
+
+    ::encode(layout.pool_ns, payload);
   }
 };
 
