@@ -36,18 +36,18 @@ private:
   ~MDentryUnlink() {}
 
 public:
-  const char *get_type_name() { return "dentry_unlink";}
-  void print(ostream& o) {
+  const char *get_type_name() const { return "dentry_unlink";}
+  void print(ostream& o) const {
     o << "dentry_unlink(" << dirfrag << " " << dn << ")";
   }
   
-  void decode_payload(CephContext *cct) {
+  void decode_payload() {
     bufferlist::iterator p = payload.begin();
     ::decode(dirfrag, p);
     ::decode(dn, p);
     ::decode(straybl, p);
   }
-  void encode_payload(CephContext *cct) {
+  void encode_payload(uint64_t features) {
     ::encode(dirfrag, payload);
     ::encode(dn, payload);
     ::encode(straybl, payload);
