@@ -20,8 +20,6 @@
 #include "mds/MDSMap.h"
 #include "include/ceph_features.h"
 
-#include <uuid/uuid.h>
-
 class MMDSMap : public Message {
  public:
   /*
@@ -61,7 +59,7 @@ class MMDSMap : public Message {
     Message(CEPH_MSG_MDS_MAP),
     fsid(f) {
     epoch = mm->get_epoch();
-    mm->encode(encoded);
+    mm->encode(encoded, -1);  // we will reencode with fewer features as necessary
   }
 private:
   ~MMDSMap() {}
@@ -82,12 +80,13 @@ public:
   void encode_payload(uint64_t features) {
     ::encode(fsid, payload);
     ::encode(epoch, payload);
-    if ((features & CEPH_FEATURE_PGID64) == 0) {
+    if ((features & CEPH_FEATURE_PGID64) == 0 ||
+	(features & CEPH_FEATURE_MDSENC) == 0) {
       // reencode for old clients.
       MDSMap m;
       m.decode(encoded);
       encoded.clear();
-      m.encode_client_old(encoded);
+      m.encode(encoded, features);
     }
     ::encode(encoded, payload);
   }
