@@ -19,7 +19,7 @@
 
 class MPGStatsAck : public Message {
 public:
-  map<pg_t,eversion_t> pg_stat;
+  map<pg_t,pair<version_t,epoch_t> > pg_stat;
   
   MPGStatsAck() : Message(MSG_PGSTATSACK) {}
 
@@ -27,15 +27,15 @@ private:
   ~MPGStatsAck() {}
 
 public:
-  const char *get_type_name() { return "pg_stats_ack"; }
-  void print(ostream& out) {
+  const char *get_type_name() const { return "pg_stats_ack"; }
+  void print(ostream& out) const {
     out << "pg_stats_ack(" << pg_stat.size() << " pgs tid " << get_tid() << ")";
   }
 
-  void encode_payload(CephContext *cct) {
+  void encode_payload(uint64_t features) {
     ::encode(pg_stat, payload);
   }
-  void decode_payload(CephContext *cct) {
+  void decode_payload() {
     bufferlist::iterator p = payload.begin();
     ::decode(pg_stat, p);
   }
