@@ -30,16 +30,16 @@ private:
   ~MRemoveSnaps() {}
 
 public:
-  const char *get_type_name() { return "remove_snaps"; }
-  void print(ostream& out) {
+  const char *get_type_name() const { return "remove_snaps"; }
+  void print(ostream& out) const {
     out << "remove_snaps(" << snaps << " v" << version << ")";
   }
 
-  void encode_payload(CephContext *cct) {
+  void encode_payload(uint64_t features) {
     paxos_encode();
     ::encode(snaps, payload);
   }
-  void decode_payload(CephContext *cct) {
+  void decode_payload() {
     bufferlist::iterator p = payload.begin();
     paxos_decode(p);
     ::decode(snaps, p);
