@@ -42,8 +42,8 @@ class FileSystemCommandHandler;
 class MDSMonitor : public PaxosService {
  public:
   // mds maps
-  FSMap fsmap;          // current
-  bufferlist mdsmap_bl;   // encoded
+  FSMap fsmap;           // current
+  bufferlist fsmap_bl;   // encoded
 
   FSMap pending_fsmap;  // current + pending updates
 
