@@ -310,10 +310,10 @@ bool MonClient::ms_dispatch(Message *m)
   return true;
 }
 
-void MonClient::send_log()
+void MonClient::send_log(bool flush)
 {
   if (log_client) {
-    Message *lm = log_client->get_mon_log_message();
+    Message *lm = log_client->get_mon_log_message(flush);
     if (lm)
       _send_mon_message(lm);
     more_log_pending = log_client->are_pending();
@@ -539,13 +539,9 @@ void MonClient::handle_auth(MAuthReply *m)
 	_send_mon_message(waiting_for_session.front());
 	waiting_for_session.pop_front();
       }
-
       _resend_mon_commands();
 
-      if (log_client) {
-	log_client->reset_session();
-	send_log();
-      }
+      send_log(true);
       if (session_established_context) {
         cb = session_established_context;
         session_established_context = NULL;
@@ -779,7 +775,9 @@ void MonClient::_renew_subs()
     m->what = sub_new;
     _send_mon_message(m);
 
-    sub_sent.insert(sub_new.begin(), sub_new.end());
+    // update sub_sent with sub_new
+    sub_new.insert(sub_sent.begin(), sub_sent.end());
+    std::swap(sub_new, sub_sent);
     sub_new.clear();
   }
 }
