@@ -274,6 +274,8 @@ int MonMap::build_initial(CephContext *cct, ostream& errout)
              << std::endl;
       return r;
     }
+    created = ceph_clock_now(cct);
+    last_changed = created;
     return 0;
   }
 
@@ -333,5 +335,7 @@ int MonMap::build_initial(CephContext *cct, ostream& errout)
     errout << "no monitors specified to connect to." << std::endl;
     return -ENOENT;
   }
+  created = ceph_clock_now(cct);
+  last_changed = created;
   return 0;
 }
