@@ -145,7 +145,7 @@ static int on_msg(struct xio_session *session,
     }
   }
 
-  return xcon->on_msg_req(session, req, more_in_batch,
+  return xcon->on_msg(session, req, more_in_batch,
 			  cb_user_context);
 }
 
@@ -583,7 +583,7 @@ enum bl_type
 #define MAX_XIO_BUF_SIZE 1044480
 
 static inline int
-xio_count_buffers(buffer::list& bl, int& req_size, int& msg_off, int& req_off)
+xio_count_buffers(const buffer::list& bl, int& req_size, int& msg_off, int& req_off)
 {
 
   const std::list<buffer::ptr>& buffers = bl.buffers();
@@ -628,7 +628,7 @@ xio_count_buffers(buffer::list& bl, int& req_size, int& msg_off, int& req_off)
 }
 
 static inline void
-xio_place_buffers(buffer::list& bl, XioMsg *xmsg, struct xio_msg*& req,
+xio_place_buffers(const buffer::list& bl, XioMsg *xmsg, struct xio_msg*& req,
 		  struct xio_iovec_ex*& msg_iov, int& req_size,
 		  int ex_cnt, int& msg_off, int& req_off, bl_type type)
 {
