@@ -137,6 +137,22 @@ int ECUtil::encode(
   return 0;
 }
 
+void ECUtil::HashInfo::append(uint64_t old_size,
+			      map<int, bufferlist> &to_append) {
+  assert(to_append.size() == cumulative_shard_hashes.size());
+  assert(old_size == total_chunk_size);
+  uint64_t size_to_append = to_append.begin()->second.length();
+  for (map<int, bufferlist>::iterator i = to_append.begin();
+       i != to_append.end();
+       ++i) {
+    assert(size_to_append == i->second.length());
+    assert((unsigned)i->first < cumulative_shard_hashes.size());
+    uint32_t new_hash = i->second.crc32c(cumulative_shard_hashes[i->first]);
+    cumulative_shard_hashes[i->first] = new_hash;
+  }
+  total_chunk_size += size_to_append;
+}
+
 void ECUtil::HashInfo::encode(bufferlist &bl) const
 {
   ENCODE_START(1, 1, bl);
