@@ -67,6 +67,7 @@ void PGLog::IndexedLog::split_into(
   PGLog::IndexedLog *olog)
 {
   list<pg_log_entry_t> oldlog;
+  unindex();
   oldlog.swap(log);
 
   eversion_t old_tail;
@@ -177,6 +178,9 @@ void PGLog::trim(
     /* If we are trimming, we must be complete up to trim_to, time
      * to throw out any divergent_priors
      */
+    if (!divergent_priors.empty()) {
+      dirty_divergent_priors = true;
+    }
     divergent_priors.clear();
     // We shouldn't be trimming the log past last_complete
     assert(trim_to <= info.last_complete);
@@ -929,8 +933,10 @@ void PGLog::read_log(ObjectStore *store, coll_t pg_coll,
 		     IndexedLog &log,
 		     pg_missing_t &missing,
 		     ostringstream &oss,
+		     bool tolerate_divergent_missing_log,
 		     const DoutPrefixProvider *dpp,
 		     set<string> *log_keys_debug)
+		   
 {
   ldpp_dout(dpp, 20) << "read_log coll " << pg_coll
 		     << " log_oid " << log_oid << dendl;
@@ -1044,7 +1050,20 @@ void PGLog::read_log(ObjectStore *store, coll_t pg_coll,
 	 * version would not have been recovered, and a newer version
 	 * would show up in the log above.
 	 */
-	assert(oi.version == i->first);
+	/**
+	 * Unfortunately the assessment above is incorrect because of
+	 * http://tracker.ceph.com/issues/17916 (we were incorrectly
+	 * not removing the divergent_priors set from disk state!),
+	 * so let's check that if the user asked us to.
+	 */
+	if (oi.version > i->first && tolerate_divergent_missing_log) {
+	  ldpp_dout(dpp, 0) << "read_log divergent_priors entry (" << *i
+			    << ") inconsistent with disk state (" <<  oi
+			    << "), assuming it is tracker.ceph.com/issues/17916"
+			    << dendl;
+	} else {
+	  assert(oi.version == i->first);
+	}
       } else {
 	ldpp_dout(dpp, 15) << "read_log  missing " << *i << dendl;
 	missing.add(i->second, i->first, eversion_t());
