@@ -32,7 +32,8 @@ int RGWMongoose::write_data(const char *buf, int len)
 RGWMongoose::RGWMongoose(mg_connection *_conn)
   : conn(_conn), status_num(0), header_done(false),
     sent_header(false), has_content_length(false),
-    explicit_keepalive(false), explicit_conn_close(false)
+    explicit_keepalive(false), explicit_conn_close(false),
+    got_eof_on_read(false)
 {
     sockaddr *lsa = mg_get_local_addr(conn);
     switch(lsa->sa_family) {
@@ -49,8 +50,21 @@ RGWMongoose::RGWMongoose(mg_connection *_conn)
 
 int RGWMongoose::read_data(char *buf, int len)
 {
-  const int ret = mg_read(conn, buf, len);
-  return (ret >= 0) ? ret : -EIO;
+  int c, ret;
+  if (got_eof_on_read) {
+    return 0;
+  }
+  for (c = 0; c < len; c += ret) {
+    ret = mg_read(conn, buf+c, len-c);
+    if (ret < 0) {
+      return -EIO;
+    }
+    if (!ret) {
+      got_eof_on_read = true;
+      break;
+    }
+  }
+  return c;
 }
 
 void RGWMongoose::flush()
@@ -103,17 +117,23 @@ int RGWMongoose::complete_request()
   return 0;
 }
 
-void RGWMongoose::init_env(CephContext *cct)
+int RGWMongoose::init_env(CephContext *cct)
 {
   env.init(cct);
   struct mg_request_info *info = mg_get_request_info(conn);
 
-  if (!info)
-    return;
+  if (!info) {
+    return -EINVAL;
+  }
 
   for (int i = 0; i < info->num_headers; i++) {
     struct mg_request_info::mg_header *header = &info->http_headers[i];
 
+    if (header->name == nullptr || header->value == nullptr) {
+      lderr(cct) << "client supplied malformed headers" << dendl;
+      return -EINVAL;
+    }
+
     if (strcasecmp(header->name, "content-length") == 0) {
       env.set("CONTENT_LENGTH", header->value);
       continue;
@@ -165,6 +185,7 @@ void RGWMongoose::init_env(CephContext *cct)
   if (info->is_ssl) {
     env.set("SERVER_PORT_SECURE", port_buf);
   }
+  return 0;
 }
 
 int RGWMongoose::send_status(int status, const char *status_name)
