@@ -22,7 +22,7 @@ class RGWFCGX : public RGWStreamIO
   int status_num;
 
 protected:
-  void init_env(CephContext *cct);
+  int init_env(CephContext *cct) override;
   int write_data(const char *buf, int len);
   int read_data(char *buf, int len);
 
