@@ -1,10 +1,12 @@
-//  Copyright (c) 2013, Facebook, Inc.  All rights reserved.
+//  Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
 //  This source code is licensed under the BSD-style license found in the
 //  LICENSE file in the root directory of this source tree. An additional grant
 //  of patent rights can be found in the PATENTS file in the same directory.
 
 #ifndef ROCKSDB_LITE
 
+#include "db/managed_iterator.h"
+
 #include <limits>
 #include <string>
 #include <utility>
@@ -13,12 +15,10 @@
 #include "db/db_impl.h"
 #include "db/db_iter.h"
 #include "db/dbformat.h"
-#include "db/managed_iterator.h"
 #include "rocksdb/env.h"
 #include "rocksdb/slice.h"
 #include "rocksdb/slice_transform.h"
-#include "table/merger.h"
-#include "util/xfunc.h"
+#include "table/merging_iterator.h"
 
 namespace rocksdb {
 
@@ -40,8 +40,6 @@ class MILock {
   }
   ~MILock() {
     this->mu_->unlock();
-    XFUNC_TEST("managed_xftest_release", "managed_unlock", managed_unlock1,
-               xf_manage_release, mi_);
   }
   ManagedIterator* GetManagedIterator() { return mi_; }
 
@@ -77,13 +75,11 @@ ManagedIterator::ManagedIterator(DBImpl* db, const ReadOptions& read_options,
       release_supported_(true) {
   read_options_.managed = false;
   if ((!read_options_.tailing) && (read_options_.snapshot == nullptr)) {
-    assert(read_options_.snapshot = db_->GetSnapshot());
+    assert(nullptr != (read_options_.snapshot = db_->GetSnapshot()));
     snapshot_created_ = true;
   }
   cfh_.SetCFD(cfd);
   mutable_iter_ = unique_ptr<Iterator>(db->NewIterator(read_options_, &cfh_));
-  XFUNC_TEST("managed_xftest_dropold", "managed_create", xf_managed_create1,
-             xf_manage_create, this);
 }
 
 ManagedIterator::~ManagedIterator() {
@@ -120,6 +116,16 @@ void ManagedIterator::Seek(const Slice& user_key) {
   SeekInternal(user_key, false);
 }
 
+void ManagedIterator::SeekForPrev(const Slice& user_key) {
+  MILock l(&in_use_, this);
+  if (NeedToRebuild()) {
+    RebuildIterator();
+  }
+  assert(mutable_iter_ != nullptr);
+  mutable_iter_->SeekForPrev(user_key);
+  UpdateCurrent();
+}
+
 void ManagedIterator::SeekInternal(const Slice& user_key, bool seek_to_first) {
   if (NeedToRebuild()) {
     RebuildIterator();
@@ -190,12 +196,12 @@ void ManagedIterator::Next() {
 
 Slice ManagedIterator::key() const {
   assert(valid_);
-  return cached_key_.GetKey();
+  return cached_key_.GetUserKey();
 }
 
 Slice ManagedIterator::value() const {
   assert(valid_);
-  return cached_value_.GetKey();
+  return cached_value_.GetUserKey();
 }
 
 Status ManagedIterator::status() const { return status_; }
@@ -208,14 +214,15 @@ void ManagedIterator::RebuildIterator() {
 void ManagedIterator::UpdateCurrent() {
   assert(mutable_iter_ != nullptr);
 
-  if (!(valid_ = mutable_iter_->Valid())) {
+  valid_ = mutable_iter_->Valid();
+  if (!valid_) {
     status_ = mutable_iter_->status();
     return;
   }
 
   status_ = Status::OK();
-  cached_key_.SetKey(mutable_iter_->key());
-  cached_value_.SetKey(mutable_iter_->value());
+  cached_key_.SetUserKey(mutable_iter_->key());
+  cached_value_.SetUserKey(mutable_iter_->value());
 }
 
 void ManagedIterator::ReleaseIter(bool only_old) {
@@ -248,8 +255,6 @@ bool ManagedIterator::TryLock() { return in_use_.try_lock(); }
 
 void ManagedIterator::UnLock() {
   in_use_.unlock();
-  XFUNC_TEST("managed_xftest_release", "managed_unlock", managed_unlock1,
-             xf_manage_release, this);
 }
 
 }  // namespace rocksdb
