@@ -1,4 +1,4 @@
-// Copyright (c) 2014, Facebook, Inc.  All rights reserved.
+// Copyright (c) 2011-present, Facebook, Inc.  All rights reserved.
 // This source code is licensed under the BSD-style license found in the
 // LICENSE file in the root directory of this source tree. An additional grant
 // of patent rights can be found in the PATENTS file in the same directory.
@@ -7,43 +7,40 @@
 // calling c++ rocksdb::WriteBatch methods from Java side.
 #include <memory>
 
+#include "db/memtable.h"
+#include "db/write_batch_internal.h"
 #include "include/org_rocksdb_WriteBatch.h"
 #include "include/org_rocksdb_WriteBatch_Handler.h"
-#include "rocksjni/portal.h"
-#include "rocksjni/writebatchhandlerjnicallback.h"
 #include "rocksdb/db.h"
-#include "rocksdb/immutable_options.h"
-#include "db/memtable.h"
-#include "rocksdb/write_batch.h"
-#include "rocksdb/status.h"
-#include "db/write_batch_internal.h"
-#include "db/writebuffer.h"
 #include "rocksdb/env.h"
 #include "rocksdb/memtablerep.h"
+#include "rocksdb/status.h"
+#include "rocksdb/write_batch.h"
+#include "rocksdb/write_buffer_manager.h"
+#include "rocksjni/portal.h"
+#include "rocksjni/writebatchhandlerjnicallback.h"
 #include "table/scoped_arena_iterator.h"
 #include "util/logging.h"
-#include "util/testharness.h"
 
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    newWriteBatch
- * Signature: (I)V
+ * Signature: (I)J
  */
-void Java_org_rocksdb_WriteBatch_newWriteBatch(
-    JNIEnv* env, jobject jobj, jint jreserved_bytes) {
-  rocksdb::WriteBatch* wb = new rocksdb::WriteBatch(
-      static_cast<size_t>(jreserved_bytes));
-
-  rocksdb::WriteBatchJni::setHandle(env, jobj, wb);
+jlong Java_org_rocksdb_WriteBatch_newWriteBatch(
+    JNIEnv* env, jclass jcls, jint jreserved_bytes) {
+  auto* wb = new rocksdb::WriteBatch(static_cast<size_t>(jreserved_bytes));
+  return reinterpret_cast<jlong>(wb);
 }
 
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    count0
- * Signature: ()I
+ * Signature: (J)I
  */
-jint Java_org_rocksdb_WriteBatch_count0(JNIEnv* env, jobject jobj) {
-  rocksdb::WriteBatch* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+jint Java_org_rocksdb_WriteBatch_count0(JNIEnv* env, jobject jobj,
+    jlong jwb_handle) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
 
   return static_cast<jint>(wb->Count());
@@ -52,10 +49,11 @@ jint Java_org_rocksdb_WriteBatch_count0(JNIEnv* env, jobject jobj) {
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    clear0
- * Signature: ()V
+ * Signature: (J)V
  */
-void Java_org_rocksdb_WriteBatch_clear0(JNIEnv* env, jobject jobj) {
-  rocksdb::WriteBatch* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+void Java_org_rocksdb_WriteBatch_clear0(JNIEnv* env, jobject jobj,
+    jlong jwb_handle) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
 
   wb->Clear();
@@ -63,14 +61,45 @@ void Java_org_rocksdb_WriteBatch_clear0(JNIEnv* env, jobject jobj) {
 
 /*
  * Class:     org_rocksdb_WriteBatch
- * Method:    put
- * Signature: ([BI[BI)V
+ * Method:    setSavePoint0
+ * Signature: (J)V
  */
-void Java_org_rocksdb_WriteBatch_put___3BI_3BI(
-    JNIEnv* env, jobject jobj,
+void Java_org_rocksdb_WriteBatch_setSavePoint0(
+    JNIEnv* env, jobject jobj, jlong jwb_handle) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
+  assert(wb != nullptr);
+
+  wb->SetSavePoint();
+}
+
+/*
+ * Class:     org_rocksdb_WriteBatch
+ * Method:    rollbackToSavePoint0
+ * Signature: (J)V
+ */
+void Java_org_rocksdb_WriteBatch_rollbackToSavePoint0(
+    JNIEnv* env, jobject jobj, jlong jwb_handle) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
+  assert(wb != nullptr);
+
+  auto s = wb->RollbackToSavePoint();
+
+  if (s.ok()) {
+    return;
+  }
+  rocksdb::RocksDBExceptionJni::ThrowNew(env, s);
+}
+
+/*
+ * Class:     org_rocksdb_WriteBatch
+ * Method:    put
+ * Signature: (J[BI[BI)V
+ */
+void Java_org_rocksdb_WriteBatch_put__J_3BI_3BI(
+    JNIEnv* env, jobject jobj, jlong jwb_handle,
     jbyteArray jkey, jint jkey_len,
     jbyteArray jentry_value, jint jentry_value_len) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto put = [&wb] (rocksdb::Slice key, rocksdb::Slice value) {
     wb->Put(key, value);
@@ -82,13 +111,13 @@ void Java_org_rocksdb_WriteBatch_put___3BI_3BI(
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    put
- * Signature: ([BI[BIJ)V
+ * Signature: (J[BI[BIJ)V
  */
-void Java_org_rocksdb_WriteBatch_put___3BI_3BIJ(
-    JNIEnv* env, jobject jobj,
+void Java_org_rocksdb_WriteBatch_put__J_3BI_3BIJ(
+    JNIEnv* env, jobject jobj, jlong jwb_handle,
     jbyteArray jkey, jint jkey_len,
     jbyteArray jentry_value, jint jentry_value_len, jlong jcf_handle) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto* cf_handle = reinterpret_cast<rocksdb::ColumnFamilyHandle*>(jcf_handle);
   assert(cf_handle != nullptr);
@@ -102,13 +131,13 @@ void Java_org_rocksdb_WriteBatch_put___3BI_3BIJ(
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    merge
- * Signature: ([BI[BI)V
+ * Signature: (J[BI[BI)V
  */
-void Java_org_rocksdb_WriteBatch_merge___3BI_3BI(
-    JNIEnv* env, jobject jobj,
+void Java_org_rocksdb_WriteBatch_merge__J_3BI_3BI(
+    JNIEnv* env, jobject jobj, jlong jwb_handle,
     jbyteArray jkey, jint jkey_len,
     jbyteArray jentry_value, jint jentry_value_len) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto merge = [&wb] (rocksdb::Slice key, rocksdb::Slice value) {
     wb->Merge(key, value);
@@ -120,13 +149,13 @@ void Java_org_rocksdb_WriteBatch_merge___3BI_3BI(
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    merge
- * Signature: ([BI[BIJ)V
+ * Signature: (J[BI[BIJ)V
  */
-void Java_org_rocksdb_WriteBatch_merge___3BI_3BIJ(
-    JNIEnv* env, jobject jobj,
+void Java_org_rocksdb_WriteBatch_merge__J_3BI_3BIJ(
+    JNIEnv* env, jobject jobj, jlong jwb_handle,
     jbyteArray jkey, jint jkey_len,
     jbyteArray jentry_value, jint jentry_value_len, jlong jcf_handle) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto* cf_handle = reinterpret_cast<rocksdb::ColumnFamilyHandle*>(jcf_handle);
   assert(cf_handle != nullptr);
@@ -140,12 +169,12 @@ void Java_org_rocksdb_WriteBatch_merge___3BI_3BIJ(
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    remove
- * Signature: ([BI)V
+ * Signature: (J[BI)V
  */
-void Java_org_rocksdb_WriteBatch_remove___3BI(
-    JNIEnv* env, jobject jobj,
+void Java_org_rocksdb_WriteBatch_remove__J_3BI(
+    JNIEnv* env, jobject jobj, jlong jwb_handle,
     jbyteArray jkey, jint jkey_len) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto remove = [&wb] (rocksdb::Slice key) {
     wb->Delete(key);
@@ -156,12 +185,12 @@ void Java_org_rocksdb_WriteBatch_remove___3BI(
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    remove
- * Signature: ([BIJ)V
+ * Signature: (J[BIJ)V
  */
-void Java_org_rocksdb_WriteBatch_remove___3BIJ(
-    JNIEnv* env, jobject jobj,
+void Java_org_rocksdb_WriteBatch_remove__J_3BIJ(
+    JNIEnv* env, jobject jobj, jlong jwb_handle,
     jbyteArray jkey, jint jkey_len, jlong jcf_handle) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto* cf_handle = reinterpret_cast<rocksdb::ColumnFamilyHandle*>(jcf_handle);
   assert(cf_handle != nullptr);
@@ -171,14 +200,56 @@ void Java_org_rocksdb_WriteBatch_remove___3BIJ(
   rocksdb::JniUtil::k_op(remove, env, jobj, jkey, jkey_len);
 }
 
+/*
+ * Class:     org_rocksdb_WriteBatch
+ * Method:    deleteRange
+ * Signature: (J[BI[BI)V
+ */
+JNIEXPORT void JNICALL Java_org_rocksdb_WriteBatch_deleteRange__J_3BI_3BI(
+    JNIEnv*, jobject, jlong, jbyteArray, jint, jbyteArray, jint);
+
+void Java_org_rocksdb_WriteBatch_deleteRange__J_3BI_3BI(
+    JNIEnv* env, jobject jobj, jlong jwb_handle, jbyteArray jbegin_key,
+    jint jbegin_key_len, jbyteArray jend_key, jint jend_key_len) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
+  assert(wb != nullptr);
+  auto deleteRange = [&wb](rocksdb::Slice beginKey, rocksdb::Slice endKey) {
+    wb->DeleteRange(beginKey, endKey);
+  };
+  rocksdb::JniUtil::kv_op(deleteRange, env, jobj, jbegin_key, jbegin_key_len,
+                          jend_key, jend_key_len);
+}
+
+/*
+ * Class:     org_rocksdb_WriteBatch
+ * Method:    deleteRange
+ * Signature: (J[BI[BIJ)V
+ */
+void Java_org_rocksdb_WriteBatch_deleteRange__J_3BI_3BIJ(
+    JNIEnv* env, jobject jobj, jlong jwb_handle, jbyteArray jbegin_key,
+    jint jbegin_key_len, jbyteArray jend_key, jint jend_key_len,
+    jlong jcf_handle) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
+  assert(wb != nullptr);
+  auto* cf_handle = reinterpret_cast<rocksdb::ColumnFamilyHandle*>(jcf_handle);
+  assert(cf_handle != nullptr);
+  auto deleteRange = [&wb, &cf_handle](rocksdb::Slice beginKey,
+                                       rocksdb::Slice endKey) {
+    wb->DeleteRange(cf_handle, beginKey, endKey);
+  };
+  rocksdb::JniUtil::kv_op(deleteRange, env, jobj, jbegin_key, jbegin_key_len,
+                          jend_key, jend_key_len);
+}
+
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    putLogData
- * Signature: ([BI)V
+ * Signature: (J[BI)V
  */
 void Java_org_rocksdb_WriteBatch_putLogData(
-    JNIEnv* env, jobject jobj, jbyteArray jblob, jint jblob_len) {
-  auto* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+    JNIEnv* env, jobject jobj, jlong jwb_handle, jbyteArray jblob,
+    jint jblob_len) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
   auto putLogData = [&wb] (rocksdb::Slice blob) {
     wb->PutLogData(blob);
@@ -189,11 +260,11 @@ void Java_org_rocksdb_WriteBatch_putLogData(
 /*
  * Class:     org_rocksdb_WriteBatch
  * Method:    iterate
- * Signature: (J)V
+ * Signature: (JJ)V
  */
 void Java_org_rocksdb_WriteBatch_iterate(
-    JNIEnv* env , jobject jobj, jlong handlerHandle) {
-  rocksdb::WriteBatch* wb = rocksdb::WriteBatchJni::getHandle(env, jobj);
+    JNIEnv* env , jobject jobj, jlong jwb_handle, jlong handlerHandle) {
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(jwb_handle);
   assert(wb != nullptr);
 
   rocksdb::Status s = wb->Iterate(
@@ -212,19 +283,20 @@ void Java_org_rocksdb_WriteBatch_iterate(
  */
 void Java_org_rocksdb_WriteBatch_disposeInternal(
     JNIEnv* env, jobject jobj, jlong handle) {
-  delete reinterpret_cast<rocksdb::WriteBatch*>(handle);
+  auto* wb = reinterpret_cast<rocksdb::WriteBatch*>(handle);
+  assert(wb != nullptr);
+  delete wb;
 }
 
 /*
  * Class:     org_rocksdb_WriteBatch_Handler
  * Method:    createNewHandler0
- * Signature: ()V
+ * Signature: ()J
  */
-void Java_org_rocksdb_WriteBatch_00024Handler_createNewHandler0(
+jlong Java_org_rocksdb_WriteBatch_00024Handler_createNewHandler0(
     JNIEnv* env, jobject jobj) {
-  const rocksdb::WriteBatchHandlerJniCallback* h =
-    new rocksdb::WriteBatchHandlerJniCallback(env, jobj);
-  rocksdb::WriteBatchHandlerJni::setHandle(env, jobj, h);
+  auto* wbjnic = new rocksdb::WriteBatchHandlerJniCallback(env, jobj);
+  return reinterpret_cast<jlong>(wbjnic);
 }
 
 /*
@@ -234,5 +306,8 @@ void Java_org_rocksdb_WriteBatch_00024Handler_createNewHandler0(
  */
 void Java_org_rocksdb_WriteBatch_00024Handler_disposeInternal(
     JNIEnv* env, jobject jobj, jlong handle) {
-  delete reinterpret_cast<rocksdb::WriteBatchHandlerJniCallback*>(handle);
+  auto* wbjnic =
+      reinterpret_cast<rocksdb::WriteBatchHandlerJniCallback*>(handle);
+  assert(wbjnic != nullptr);
+  delete wbjnic;
 }
