@@ -5,13 +5,15 @@ package org.rocksdb;
  * {@link org.rocksdb.RocksDB}.
  */
 public class FlushOptions extends RocksObject {
+  static {
+    RocksDB.loadLibrary();
+  }
 
   /**
    * Construct a new instance of FlushOptions.
    */
   public FlushOptions(){
-    super();
-    newFlushOptions();
+    super(newFlushOptions());
   }
 
   /**
@@ -23,7 +25,7 @@ public class FlushOptions extends RocksObject {
    * @return instance of current FlushOptions.
    */
   public FlushOptions setWaitForFlush(final boolean waitForFlush) {
-    assert(isInitialized());
+    assert(isOwningHandle());
     setWaitForFlush(nativeHandle_, waitForFlush);
     return this;
   }
@@ -35,16 +37,12 @@ public class FlushOptions extends RocksObject {
    *     waits for termination of the flush process.
    */
   public boolean waitForFlush() {
-    assert(isInitialized());
+    assert(isOwningHandle());
     return waitForFlush(nativeHandle_);
   }
 
-  @Override protected void disposeInternal() {
-    disposeInternal(nativeHandle_);
-  }
-
-  private native void newFlushOptions();
-  private native void disposeInternal(long handle);
+  private native static long newFlushOptions();
+  @Override protected final native void disposeInternal(final long handle);
   private native void setWaitForFlush(long handle,
       boolean wait);
   private native boolean waitForFlush(long handle);
