@@ -25,8 +25,7 @@
 #include "common/Thread.h"
 #include "common/shared_cache.hpp"
 #include "common/ceph_argparse.h"
-#include "global/global_init.h"
-#include <gtest/gtest.h>
+#include "test/unit.h"
 
 class SharedLRUTest : public SharedLRU<unsigned int, int> {
 public:
@@ -393,17 +392,6 @@ TEST(SharedCache_all, lru) {
   ASSERT_TRUE(cache.lookup(0).get());
 }
 
-int main(int argc, char **argv) {
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
-  common_init_finish(g_ceph_context);
-
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
-}
-
 // Local Variables:
 // compile-command: "cd ../.. ; make unittest_shared_cache && ./unittest_shared_cache # --gtest_filter=*.* --log-to-stderr=true"
 // End:
