@@ -18,10 +18,10 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gtest/gtest.h>
-#include "global/global_init.h"
+#include "gtest/gtest.h"
 #include "compressor/CompressionPlugin.h"
 #include "common/ceph_argparse.h"
+#include "global/global_init.h"
 #include "global/global_context.h"
 #include "common/config.h"
 
@@ -51,7 +51,10 @@ int main(int argc, char **argv) {
   vector<const char*> args;
   argv_to_vec(argc, (const char **)argv, args);
 
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
+  auto cct = global_init(NULL, args,
+                         CEPH_ENTITY_TYPE_CLIENT,
+                         CODE_ENVIRONMENT_UTILITY,
+                         0);
   common_init_finish(g_ceph_context);
 
   const char* env = getenv("CEPH_LIB");
