@@ -20,12 +20,12 @@
 
 #include "crush/CrushWrapper.h"
 #include "include/stringify.h"
-#include "global/global_init.h"
 #include "erasure-code/jerasure/ErasureCodeJerasure.h"
-#include "common/ceph_argparse.h"
 #include "global/global_context.h"
 #include "common/config.h"
 #include "gtest/gtest.h"
+#include "test/unit.h"
+
 
 template <typename T>
 class ErasureCodeTest : public ::testing::Test {
@@ -357,22 +357,6 @@ TEST(ErasureCodeTest, create_ruleset)
   }
 }
 
-int main(int argc, char **argv)
-{
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
-  common_init_finish(g_ceph_context);
-
-  const char* env = getenv("CEPH_LIB");
-  string directory(env ? env : "lib");
-  g_conf->set_val("erasure_code_dir", directory, false, false);
-
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
-}
-
 /* 
  * Local Variables:
  * compile-command: "cd ../.. ;
