@@ -13,6 +13,7 @@
  */
 
 #include <errno.h>
+#include <stdlib.h>
 #include "arch/probe.h"
 #include "arch/intel.h"
 #include "global/global_init.h"
@@ -21,6 +22,7 @@
 #include "global/global_context.h"
 #include "common/config.h"
 #include "gtest/gtest.h"
+#include "test/unit.h"
 
 TEST(ErasureCodePlugin, factory)
 {
@@ -51,20 +53,6 @@ TEST(ErasureCodePlugin, factory)
   }
 }
 
-int main(int argc, char **argv)
-{
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
-  common_init_finish(g_ceph_context);
-
-  g_conf->set_val("erasure_code_dir", ".libs", false, false);
-
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
-}
-
 /*
  * Local Variables:
  * compile-command: "cd ../.. ; make -j4 &&
