@@ -20,6 +20,7 @@
 
 #include <errno.h>
 #include <pthread.h>
+#include <stdlib.h>
 
 #include "crush/CrushWrapper.h"
 #include "osd/osd_types.h"
@@ -30,6 +31,7 @@
 #include "common/ceph_argparse.h"
 #include "global/global_context.h"
 #include "gtest/gtest.h"
+#include "test/unit.h"
 
 void* thread1(void* pParam);
 void* thread2(void* pParam);
@@ -2666,20 +2668,6 @@ TEST(ErasureCodeShec, get_chunk_size_1_2)
   delete profile;
 }
 
-int main(int argc, char **argv)
-{
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **) argv, args);
-
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
-  common_init_finish(g_ceph_context);
-
-  g_conf->set_val("erasure_code_dir", ".libs", false, false);
-
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
-}
-
 void* thread1(void* pParam)
 {
   ErasureCodeShec* shec = (ErasureCodeShec*) pParam;
