@@ -5,6 +5,7 @@
 #define CEPH_TEST_LIBRBD_MOCK_AIO_IMAGE_REQUEST_WQ_H
 
 #include "gmock/gmock.h"
+#include "librbd/AioImageRequestWQ.h"
 
 class Context;
 
@@ -14,10 +15,7 @@ struct MockAioImageRequestWQ {
   MOCK_METHOD1(block_writes, void(Context *));
   MOCK_METHOD0(unblock_writes, void());
 
-  MOCK_METHOD0(set_require_lock_on_read, void());
-  MOCK_METHOD0(clear_require_lock_on_read, void());
-
-  MOCK_CONST_METHOD0(is_lock_request_needed, bool());
+  MOCK_METHOD2(set_require_lock, void(AioDirection, bool));
 };
 
 } // namespace librbd
