@@ -16,6 +16,7 @@ extern void register_test_journal_entries();
 extern void register_test_journal_replay();
 extern void register_test_object_map();
 extern void register_test_mirroring();
+extern void register_test_mirroring_watcher();
 #endif // TEST_LIBRBD_INTERNALS
 
 int main(int argc, char **argv)
@@ -28,6 +29,7 @@ int main(int argc, char **argv)
   register_test_journal_replay();
   register_test_object_map();
   register_test_mirroring();
+  register_test_mirroring_watcher();
 #endif // TEST_LIBRBD_INTERNALS
 
   ::testing::InitGoogleTest(&argc, argv);
