@@ -12,7 +12,7 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU Library Public License for more details.
 #
-source ../qa/workunits/ceph-helpers.sh
+source $CEPH_ROOT/qa/workunits/ceph-helpers.sh
 
 function run() {
     local dir=$1
@@ -36,7 +36,7 @@ function TEST_mon_ping() {
 
     run_mon $dir a || return 1
 
-    ./ceph ping mon.a || return 1
+    ceph ping mon.a || return 1
 }
 
 main mon-ping "$@"
