@@ -52,9 +52,11 @@
 #include "msg/async/EventKqueue.h"
 #endif
 #include "msg/async/EventSelect.h"
+#include "test/unit.h"
 
 #include <gtest/gtest.h>
 
+
 #if GTEST_HAS_PARAM_TEST
 
 class EventDriverTest : public ::testing::TestWithParam<const char*> {
@@ -144,7 +146,7 @@ void* echoclient(void *arg)
   sa.sin_family = AF_INET;
   sa.sin_port = htons(port);
   char addr[] = "127.0.0.1";
-  int r = inet_aton(addr, &sa.sin_addr);
+  int r = inet_pton(AF_INET, addr, &sa.sin_addr);
 
   int connect_sd = ::socket(AF_INET, SOCK_STREAM, 0);
   if (connect_sd >= 0) {
@@ -272,7 +274,7 @@ class Worker : public Thread {
 
  public:
   EventCenter center;
-  Worker(CephContext *c): cct(c), done(false), center(c) {
+  explicit Worker(CephContext *c): cct(c), done(false), center(c) {
     center.init(100);
   }
   void stop() {
@@ -280,7 +282,7 @@ class Worker : public Thread {
     center.wakeup();
   }
   void* entry() {
-    center.set_owner(pthread_self());
+    center.set_owner();
     while (!done)
       center.process_events(1000000);
     return 0;
@@ -307,8 +309,8 @@ TEST(EventCenterTest, DispatchTest) {
   atomic_t count(0);
   Mutex lock("DispatchTest::lock");
   Cond cond;
-  worker1.create();
-  worker2.create();
+  worker1.create("worker_1");
+  worker2.create("worker_2");
   for (int i = 0; i < 10000; ++i) {
     count.inc();
     worker1.center.dispatch_event_external(EventCallbackRef(new CountEvent(&count, &lock, &cond)));
@@ -349,17 +351,6 @@ TEST(DummyTest, ValueParameterizedTestsAreNotSupportedOnThisPlatform) {}
 #endif
 
 
-int main(int argc, char **argv) {
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
-  common_init_finish(g_ceph_context);
-
-  ::testing::InitGoogleTest(&argc, argv);
-  return RUN_ALL_TESTS();
-}
-
 /*
  * Local Variables:
  * compile-command: "cd ../.. ; make ceph_test_async_driver && 
