@@ -196,6 +196,11 @@ EOF
     jq -c '.inconsistents | sort' $dir/json > $dir/csjson
     diff $dir/csjson $dir/checkcsjson || return 1
 
+    if which jsonschema > /dev/null;
+    then
+      jsonschema -i $dir/json $CEPH_ROOT/doc/rados/command/list-inconsistent-snap.json || return 1
+    fi
+
     for i in `seq 1 7`
     do
         rados -p $poolname rmsnap snap$i
