@@ -41,6 +41,7 @@ const char *parse_good[] = {
   "allow rwx pool foo; allow r pool bar",
   "allow auid 123 rwx",
   "allow pool foo rwx, allow pool bar r",
+  "allow pool foo.froo.foo rwx, allow pool bar r",
   "allow pool foo rwx ; allow pool bar r",
   "allow pool foo rwx ;allow pool bar r",
   "allow pool foo rwx; allow pool bar r",
@@ -59,6 +60,7 @@ const char *parse_good[] = {
   "  allow pool foo rwx; allow pool bar r  ",
   "  allow     pool foo rwx; allow pool bar r  ",
   " allow wx pool taco",
+  "\tallow\nwx\tpool \n taco\t",
   "allow r   pool    foo    object_prefix   blah   ;   allow   w   auid  5",
   "allow class-read object_prefix rbd_children, allow pool libvirt-pool-test rwx",
   "allow class-read object_prefix rbd-children, allow pool libvirt_pool_test rwx",
