@@ -22,6 +22,7 @@ class CrossProcessSem;
 /*
  * st_rados_create_pool
  *
+ * Waits, then posts to setup_sem.
  * Creates a pool and populates it with some objects.
  * Then, calls pool_setup_sem->post()
  */
@@ -30,15 +31,23 @@ class StRadosCreatePool : public SysTestRunnable
 public:
   static std::string get_random_buf(int sz);
   StRadosCreatePool(int argc, const char **argv,
-	CrossProcessSem *pool_setup_sem, CrossProcessSem *close_create_pool_sem,
-	int num_objects, const std::string &suffix);
+		    CrossProcessSem *setup_sem,
+		    CrossProcessSem *pool_setup_sem,
+		    CrossProcessSem *close_create_pool_sem,
+		    const std::string &pool_name,
+		    int num_objects,
+		    const std::string &suffix);
   ~StRadosCreatePool();
   virtual int run();
 private:
+  CrossProcessSem *m_setup_sem;
   CrossProcessSem *m_pool_setup_sem;
   CrossProcessSem *m_close_create_pool;
+  std::string m_pool_name;
   int m_num_objects;
   std::string m_suffix;
 };
 
+std::string get_temp_pool_name(const char* prefix);
+
 #endif
