@@ -8,11 +8,11 @@
 #include "global/global_init.h"
 #include "common/config.h"
 #include "common/Finisher.h"
-#include "os/FileJournal.h"
+#include "os/filestore/FileJournal.h"
 #include "include/Context.h"
 #include "common/Mutex.h"
 #include "common/safe_io.h"
-#include "os/JournalingObjectStore.h"
+#include "os/filestore/JournalingObjectStore.h"
 
 Finisher *finisher;
 Cond sync_cond;
@@ -70,7 +70,8 @@ int main(int argc, char **argv) {
   vector<const char*> args;
   argv_to_vec(argc, (const char **)argv, args);
 
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
+  auto cct = global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT,
+			 CODE_ENVIRONMENT_UTILITY, 0);
   common_init_finish(g_ceph_context);
 
   char mb[10];
@@ -138,10 +139,11 @@ TEST(TestFileJournal, WriteSmall) {
     ASSERT_EQ(0, j.create());
     j.make_writeable();
 
-    list<ObjectStore::Transaction*> tls;
+    vector<ObjectStore::Transaction> tls;
     bufferlist bl;
     bl.append("small");
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(1, bl, orig_len, new C_SafeCond(&wait_lock, &cond, &done));
     wait();
 
@@ -168,8 +170,9 @@ TEST(TestFileJournal, WriteBig) {
       memset(foo, 1, sizeof(foo));
       bl.append(foo, sizeof(foo));
     }
-    list<ObjectStore::Transaction*> tls;
+    vector<ObjectStore::Transaction> tls;
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(1, bl, orig_len, new C_SafeCond(&wait_lock, &cond, &done));
     wait();
     j.close();
@@ -191,13 +194,14 @@ TEST(TestFileJournal, WriteMany) {
 
     C_GatherBuilder gb(g_ceph_context, new C_SafeCond(&wait_lock, &cond, &done));
 
-    list<ObjectStore::Transaction*> tls;
+    vector<ObjectStore::Transaction> tls;
     bufferlist bl;
     bl.append("small");
     uint64_t seq = 1;
     for (int i=0; i<100; i++) {
       bl.append("small");
       int orig_len = j.prepare_entry(tls, &bl);
+      j.reserve_throttle_and_backoff(bl.length());
       j.submit_entry(seq++, bl, orig_len, gb.new_sub());
     }
     gb.activate();
@@ -225,8 +229,9 @@ TEST(TestFileJournal, WriteManyVecs) {
 
     bufferlist first;
     first.append("small");
-    list<ObjectStore::Transaction*> tls;
+    vector<ObjectStore::Transaction> tls;
     int orig_len = j.prepare_entry(tls, &first);
+    j.reserve_throttle_and_backoff(first.length());
     j.submit_entry(1, first, orig_len, gb.new_sub());
 
     bufferlist bl;
@@ -237,6 +242,7 @@ TEST(TestFileJournal, WriteManyVecs) {
     }
     bufferlist origbl = bl;
     orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(2, bl, orig_len, gb.new_sub());
     gb.activate();
     wait();
@@ -261,7 +267,7 @@ TEST(TestFileJournal, ReplaySmall) {
   g_ceph_context->_conf->set_val("journal_write_header_frequency", "0");
   g_ceph_context->_conf->apply_changes(NULL);
 
-  list<ObjectStore::Transaction*> tls;
+  vector<ObjectStore::Transaction> tls;
 
   for (unsigned i = 0 ; i < 3; ++i) {
     SCOPED_TRACE(subtests[i].description);
@@ -276,12 +282,15 @@ TEST(TestFileJournal, ReplaySmall) {
     bufferlist bl;
     bl.append("small");
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(1, bl, orig_len, gb.new_sub());
     bl.append("small");
     orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(2, bl, orig_len, gb.new_sub());
     bl.append("small");
     orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(3, bl, orig_len, gb.new_sub());
     gb.activate();
     wait();
@@ -319,7 +328,7 @@ TEST(TestFileJournal, ReplayCorrupt) {
   g_ceph_context->_conf->set_val("journal_write_header_frequency", "0");
   g_ceph_context->_conf->apply_changes(NULL);
 
-  list<ObjectStore::Transaction*> tls;
+  vector<ObjectStore::Transaction> tls;
   for (unsigned i = 0 ; i < 3; ++i) {
     SCOPED_TRACE(subtests[i].description);
     fsid.generate_random();
@@ -335,15 +344,19 @@ TEST(TestFileJournal, ReplayCorrupt) {
     bufferlist bl;
     bl.append(needle);
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(1, bl, orig_len, gb.new_sub());
     bl.append(needle);
     orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(2, bl, orig_len, gb.new_sub());
     bl.append(needle);
     orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(3, bl, orig_len, gb.new_sub());
     bl.append(needle);
     orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(4, bl, orig_len, gb.new_sub());
     gb.activate();
     wait();
@@ -416,7 +429,7 @@ TEST(TestFileJournal, WriteTrim) {
     memset(foo, 1, sizeof(foo));
 
     uint64_t seq = 1, committed = 0;
-    list<ObjectStore::Transaction*> tls;
+    vector<ObjectStore::Transaction> tls;
 
     for (unsigned i=0; i<size_mb*2; i++) {
       bl.clear();
@@ -424,6 +437,7 @@ TEST(TestFileJournal, WriteTrim) {
       bl.zero();
       ls.push_back(new C_Sync);
       int orig_len = j.prepare_entry(tls, &bl);
+      j.reserve_throttle_and_backoff(bl.length());
       j.submit_entry(seq++, bl, orig_len, ls.back()->c);
 
       while (ls.size() > size_mb/2) {
@@ -450,7 +464,7 @@ TEST(TestFileJournal, WriteTrimSmall) {
   g_ceph_context->_conf->set_val("journal_ignore_corruption", "false");
   g_ceph_context->_conf->set_val("journal_write_header_frequency", "0");
   g_ceph_context->_conf->apply_changes(NULL);
-  list<ObjectStore::Transaction*> tls;
+  vector<ObjectStore::Transaction> tls;
 
   for (unsigned i = 0 ; i < 3; ++i) {
     SCOPED_TRACE(subtests[i].description);
@@ -475,6 +489,7 @@ TEST(TestFileJournal, WriteTrimSmall) {
       bl.zero();
       ls.push_back(new C_Sync);
       int orig_len = j.prepare_entry(tls, &bl);
+      j.reserve_throttle_and_backoff(bl.length());
       j.submit_entry(seq++, bl, orig_len, ls.back()->c);
 
       while (ls.size() > size_mb/2) {
@@ -500,7 +515,7 @@ TEST(TestFileJournal, ReplayDetectCorruptFooterMagic) {
   g_ceph_context->_conf->set_val("journal_write_header_frequency", "1");
   g_ceph_context->_conf->apply_changes(NULL);
 
-  list<ObjectStore::Transaction*> tls;
+  vector<ObjectStore::Transaction> tls;
   for (unsigned i = 0 ; i < 3; ++i) {
     SCOPED_TRACE(subtests[i].description);
     fsid.generate_random();
@@ -516,6 +531,7 @@ TEST(TestFileJournal, ReplayDetectCorruptFooterMagic) {
       bufferlist bl;
       bl.append(needle);
       int orig_len = j.prepare_entry(tls, &bl);
+      j.reserve_throttle_and_backoff(bl.length());
       j.submit_entry(i, bl, orig_len, gb.new_sub());
     }
     gb.activate();
@@ -524,6 +540,7 @@ TEST(TestFileJournal, ReplayDetectCorruptFooterMagic) {
     bufferlist bl;
     bl.append("needle");
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(5, bl, orig_len, new C_SafeCond(&wait_lock, &cond, &done));
     wait();
 
@@ -557,7 +574,7 @@ TEST(TestFileJournal, ReplayDetectCorruptPayload) {
   g_ceph_context->_conf->set_val("journal_write_header_frequency", "1");
   g_ceph_context->_conf->apply_changes(NULL);
 
-  list<ObjectStore::Transaction*> tls;
+  vector<ObjectStore::Transaction> tls;
   for (unsigned i = 0 ; i < 3; ++i) {
     SCOPED_TRACE(subtests[i].description);
     fsid.generate_random();
@@ -573,6 +590,7 @@ TEST(TestFileJournal, ReplayDetectCorruptPayload) {
       bufferlist bl;
       bl.append(needle);
       int orig_len = j.prepare_entry(tls, &bl);
+      j.reserve_throttle_and_backoff(bl.length());
       j.submit_entry(i, bl, orig_len, gb.new_sub());
     }
     gb.activate();
@@ -581,6 +599,7 @@ TEST(TestFileJournal, ReplayDetectCorruptPayload) {
     bufferlist bl;
     bl.append("needle");
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(5, bl, orig_len, new C_SafeCond(&wait_lock, &cond, &done));
     wait();
 
@@ -614,7 +633,7 @@ TEST(TestFileJournal, ReplayDetectCorruptHeader) {
   g_ceph_context->_conf->set_val("journal_write_header_frequency", "1");
   g_ceph_context->_conf->apply_changes(NULL);
 
-  list<ObjectStore::Transaction*> tls;
+  vector<ObjectStore::Transaction> tls;
   for (unsigned i = 0 ; i < 3; ++i) {
     SCOPED_TRACE(subtests[i].description);
     fsid.generate_random();
@@ -630,6 +649,7 @@ TEST(TestFileJournal, ReplayDetectCorruptHeader) {
       bufferlist bl;
       bl.append(needle);
       int orig_len = j.prepare_entry(tls, &bl);
+      j.reserve_throttle_and_backoff(bl.length());
       j.submit_entry(i, bl, orig_len, gb.new_sub());
     }
     gb.activate();
@@ -638,6 +658,7 @@ TEST(TestFileJournal, ReplayDetectCorruptHeader) {
     bufferlist bl;
     bl.append("needle");
     int orig_len = j.prepare_entry(tls, &bl);
+    j.reserve_throttle_and_backoff(bl.length());
     j.submit_entry(5, bl, orig_len, new C_SafeCond(&wait_lock, &cond, &done));
     wait();
 
