@@ -1,10 +1,8 @@
 #include "gtest/gtest.h"
 
 #include "common/WorkQueue.h"
-#include "global/global_context.h"
 #include "common/ceph_argparse.h"
-#include "global/global_init.h"
-#include "common/common_init.h"
+#include "test/unit.h"
 
 TEST(WorkQueue, StartStop)
 {
@@ -56,17 +54,3 @@ TEST(WorkQueue, Resize)
   sleep(1);
   tp.stop();
 }
-
-
-int main(int argc, char **argv)
-{
-  ::testing::InitGoogleTest(&argc, argv);
-
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
-  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
-  common_init_finish(g_ceph_context);
-
-  return RUN_ALL_TESTS();
-}
