@@ -15,21 +15,6 @@ namespace clone {
 namespace at = argument_types;
 namespace po = boost::program_options;
 
-int do_clone(librbd::RBD &rbd, librados::IoCtx &p_ioctx,
-             const char *p_name, const char *p_snapname,
-             librados::IoCtx &c_ioctx, const char *c_name,
-             librbd::ImageOptions& opts) {
-  uint64_t features;
-  int r = opts.get(RBD_IMAGE_OPTION_FEATURES, &features);
-  assert(r == 0);
-
-  if ((features & RBD_FEATURE_LAYERING) != RBD_FEATURE_LAYERING) {
-    return -EINVAL;
-  }
-
-  return rbd.clone3(p_ioctx, p_name, p_snapname, c_ioctx, c_name, opts);
-}
-
 void get_arguments(po::options_description *positional,
                    po::options_description *options) {
   at::add_snap_spec_options(positional, options, at::ARGUMENT_MODIFIER_SOURCE);
@@ -79,9 +64,18 @@ int execute(const po::variables_map &vm) {
     return r;
   }
 
+  uint64_t features;
+  r = opts.get(RBD_IMAGE_OPTION_FEATURES, &features);
+  if (r != -ENOENT) {
+    if ((features & RBD_FEATURE_LAYERING) != RBD_FEATURE_LAYERING) {
+      std::cerr << "rbd: clone image must support layering" << std::endl;
+      return -EINVAL;
+    }
+  }
+
   librbd::RBD rbd;
-  r = do_clone(rbd, io_ctx, image_name.c_str(), snap_name.c_str(), dst_io_ctx,
-               dst_image_name.c_str(), opts);
+  r = rbd.clone3(io_ctx, image_name.c_str(), snap_name.c_str(), dst_io_ctx,
+		 dst_image_name.c_str(), opts);
   if (r < 0) {
     std::cerr << "rbd: clone error: " << cpp_strerror(r) << std::endl;
     return r;
