@@ -36,6 +36,9 @@ namespace mirror {
 
 namespace {
 
+const std::vector<std::string> UNIQUE_PEER_CONFIG_KEYS {
+  {"monmap", "mon_host", "key", "keyfile", "keyring"}};
+
 class ReplayerAdminSocketCommand {
 public:
   virtual ~ReplayerAdminSocketCommand() {}
@@ -270,14 +273,14 @@ int Replayer::init()
 
   int r = init_rados(g_ceph_context->_conf->cluster,
                      g_ceph_context->_conf->name.to_str(),
-                     "local cluster", &m_local_rados);
+                     "local cluster", &m_local_rados, false);
   if (r < 0) {
     return r;
   }
 
   r = init_rados(m_peer.cluster_name, m_peer.client_name,
                  std::string("remote peer ") + stringify(m_peer),
-                 &m_remote_rados);
+                 &m_remote_rados, true);
   if (r < 0) {
     return r;
   }
@@ -314,7 +317,8 @@ int Replayer::init()
 
 int Replayer::init_rados(const std::string &cluster_name,
                          const std::string &client_name,
-                         const std::string &description, RadosRef *rados_ref) {
+                         const std::string &description, RadosRef *rados_ref,
+                         bool strip_cluster_overrides) {
   rados_ref->reset(new librados::Rados());
 
   // NOTE: manually bootstrap a CephContext here instead of via
@@ -339,6 +343,23 @@ int Replayer::init_rados(const std::string &cluster_name,
     cct->put();
     return r;
   }
+
+  // preserve cluster-specific config settings before applying environment/cli
+  // overrides
+  std::map<std::string, std::string> config_values;
+  if (strip_cluster_overrides) {
+    // remote peer connections shouldn't apply cluster-specific
+    // configuration settings
+    for (auto& key : UNIQUE_PEER_CONFIG_KEYS) {
+      char* value = nullptr;
+      r = cct->_conf->get_val(key.c_str(), &value, -1);
+      if (r >= 0) {
+        config_values[key] = value;
+      }
+      free(value);
+    }
+  }
+
   cct->_conf->parse_env();
 
   // librados::Rados::conf_parse_env
@@ -364,6 +385,22 @@ int Replayer::init_rados(const std::string &cluster_name,
     }
   }
 
+  if (strip_cluster_overrides) {
+    // remote peer connections shouldn't apply cluster-specific
+    // configuration settings
+    for (auto& pair : config_values) {
+      char* value = nullptr;
+      r = cct->_conf->get_val(pair.first.c_str(), &value, -1);
+      if (r >= 0 && pair.second != std::string(value)) {
+        dout(0) << "reverting global config option override: "
+                << pair.first << ": " << value << " -> " << pair.second
+                << dendl;
+        cct->_conf->set_val_or_die(pair.first.c_str(), pair.second.c_str());
+      }
+      free(value);
+    }
+  }
+
   // disable unnecessary librbd cache
   cct->_conf->set_val_or_die("rbd_cache", "false");
   cct->_conf->apply_changes(nullptr);
