@@ -5,6 +5,7 @@
 #include "ObjectCopyRequest.h"
 #include "include/stringify.h"
 #include "common/errno.h"
+#include "common/Timer.h"
 #include "journal/Journaler.h"
 #include "librbd/Utils.h"
 #include "tools/rbd_mirror/ProgressContext.h"
@@ -29,14 +30,17 @@ ImageCopyRequest<I>::ImageCopyRequest(I *local_image_ctx, I *remote_image_ctx,
                                       MirrorPeerSyncPoint *sync_point,
                                       Context *on_finish,
 				      ProgressContext *progress_ctx)
-  : m_local_image_ctx(local_image_ctx), m_remote_image_ctx(remote_image_ctx),
+  : BaseRequest("rbd::mirror::image_sync::ImageCopyRequest",
+		local_image_ctx->cct, on_finish),
+    m_local_image_ctx(local_image_ctx), m_remote_image_ctx(remote_image_ctx),
     m_timer(timer), m_timer_lock(timer_lock), m_journaler(journaler),
     m_client_meta(client_meta), m_sync_point(sync_point),
-    m_on_finish(on_finish), m_progress_ctx(progress_ctx),
+    m_progress_ctx(progress_ctx),
     m_lock(unique_lock_name("ImageCopyRequest::m_lock", this)),
+    m_updating_sync_point(false), m_update_sync_ctx(nullptr),
+    m_update_sync_point_interval(g_ceph_context->_conf->rbd_mirror_sync_point_update_age),
     m_client_meta_copy(*client_meta) {
   assert(!m_client_meta_copy.sync_points.empty());
-  assert(!m_client_meta_copy.snap_seqs.empty());
 }
 
 template <typename I>
@@ -54,8 +58,7 @@ template <typename I>
 void ImageCopyRequest<I>::cancel() {
   Mutex::Locker locker(m_lock);
 
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << dendl;
+  dout(20) << dendl;
   m_canceled = true;
 }
 
@@ -79,8 +82,7 @@ void ImageCopyRequest<I>::send_update_max_object_count() {
 
   update_progress("UPDATE_MAX_OBJECT_COUNT");
 
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": sync_object_count=" << max_objects << dendl;
+  dout(20) << ": sync_object_count=" << max_objects << dendl;
 
   m_client_meta_copy = *m_client_meta;
   m_client_meta_copy.sync_object_count = max_objects;
@@ -97,12 +99,20 @@ void ImageCopyRequest<I>::send_update_max_object_count() {
 
 template <typename I>
 void ImageCopyRequest<I>::handle_update_max_object_count(int r) {
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": r=" << r << dendl;
+  dout(20) << ": r=" << r << dendl;
+
+  if (r == 0) {
+    Mutex::Locker locker(m_lock);
+    if (m_canceled) {
+      dout(10) << ": image copy canceled" << dendl;
+      r = -ECANCELED;
+    }
+  }
 
   if (r < 0) {
-    lderr(cct) << ": failed to update client data: " << cpp_strerror(r)
-               << dendl;
+    if (r != -ECANCELED) {
+      derr << ": failed to update client data: " << cpp_strerror(r) << dendl;
+    }
     finish(r);
     return;
   }
@@ -138,7 +148,22 @@ void ImageCopyRequest<I>::send_object_copies() {
       }
     }
     complete = (m_current_ops == 0);
+
+    if (!complete) {
+      m_update_sync_ctx = new FunctionContext([this](int r) {
+          this->send_update_sync_point();
+      });
+    }
   }
+
+  {
+    Mutex::Locker timer_locker(*m_timer_lock);
+    if (m_update_sync_ctx) {
+      m_timer->add_event_after(m_update_sync_point_interval,
+                               m_update_sync_ctx);
+    }
+  }
+
   if (complete) {
     send_flush_sync_point();
   }
@@ -147,16 +172,19 @@ void ImageCopyRequest<I>::send_object_copies() {
 template <typename I>
 void ImageCopyRequest<I>::send_next_object_copy() {
   assert(m_lock.is_locked());
-  if (m_canceled) {
-    return;
-  } else if (m_ret_val < 0 || m_object_no >= m_end_object_no) {
+
+  if (m_canceled && m_ret_val == 0) {
+    dout(10) << ": image copy canceled" << dendl;
+    m_ret_val = -ECANCELED;
+  }
+
+  if (m_ret_val < 0 || m_object_no >= m_end_object_no) {
     return;
   }
 
   uint64_t ono = m_object_no++;
 
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": object_num=" << ono << dendl;
+  dout(20) << ": object_num=" << ono << dendl;
 
   ++m_current_ops;
 
@@ -169,8 +197,7 @@ void ImageCopyRequest<I>::send_next_object_copy() {
 
 template <typename I>
 void ImageCopyRequest<I>::handle_object_copy(int r) {
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": r=" << r << dendl;
+  dout(20) << ": r=" << r << dendl;
 
   int percent;
   bool complete;
@@ -178,10 +205,11 @@ void ImageCopyRequest<I>::handle_object_copy(int r) {
     Mutex::Locker locker(m_lock);
     assert(m_current_ops > 0);
     --m_current_ops;
+
     percent = 100 * m_object_no / m_end_object_no;
 
     if (r < 0) {
-      lderr(cct) << ": object copy failed: " << cpp_strerror(r) << dendl;
+      derr << ": object copy failed: " << cpp_strerror(r) << dendl;
       if (m_ret_val == 0) {
         m_ret_val = r;
       }
@@ -194,6 +222,92 @@ void ImageCopyRequest<I>::handle_object_copy(int r) {
   update_progress("COPY_OBJECT " + stringify(percent) + "%", false);
 
   if (complete) {
+    bool do_flush = true;
+    {
+      Mutex::Locker timer_locker(*m_timer_lock);
+      Mutex::Locker locker(m_lock);
+      if (!m_updating_sync_point) {
+        if (m_update_sync_ctx != nullptr) {
+          m_timer->cancel_event(m_update_sync_ctx);
+          m_update_sync_ctx = nullptr;
+        }
+      } else {
+        do_flush = false;
+      }
+    }
+
+    if (do_flush) {
+      send_flush_sync_point();
+    }
+  }
+}
+
+template <typename I>
+void ImageCopyRequest<I>::send_update_sync_point() {
+  Mutex::Locker l(m_lock);
+
+  m_update_sync_ctx = nullptr;
+
+  if (m_canceled || m_ret_val < 0 || m_current_ops == 0) {
+    return;
+  }
+
+  if (m_sync_point->object_number &&
+      (m_object_no-1) == m_sync_point->object_number.get()) {
+    // update sync point did not progress since last sync
+    return;
+  }
+
+  m_updating_sync_point = true;
+
+  m_client_meta_copy = *m_client_meta;
+  m_sync_point->object_number = m_object_no - 1;
+
+  CephContext *cct = m_local_image_ctx->cct;
+  ldout(cct, 20) << ": sync_point=" << *m_sync_point << dendl;
+
+  bufferlist client_data_bl;
+  librbd::journal::ClientData client_data(*m_client_meta);
+  ::encode(client_data, client_data_bl);
+
+  Context *ctx = create_context_callback<
+    ImageCopyRequest<I>, &ImageCopyRequest<I>::handle_update_sync_point>(
+      this);
+  m_journaler->update_client(client_data_bl, ctx);
+}
+
+template <typename I>
+void ImageCopyRequest<I>::handle_update_sync_point(int r) {
+  CephContext *cct = m_local_image_ctx->cct;
+  ldout(cct, 20) << ": r=" << r << dendl;
+
+  if (r < 0) {
+    *m_client_meta = m_client_meta_copy;
+    lderr(cct) << ": failed to update client data: " << cpp_strerror(r)
+               << dendl;
+  }
+
+  bool complete;
+  {
+    Mutex::Locker l(m_lock);
+    m_updating_sync_point = false;
+
+    complete = m_current_ops == 0 || m_canceled || m_ret_val < 0;
+
+    if (!complete) {
+      m_update_sync_ctx = new FunctionContext([this](int r) {
+          this->send_update_sync_point();
+      });
+    }
+  }
+
+  if (!complete) {
+    Mutex::Locker timer_lock(*m_timer_lock);
+    if (m_update_sync_ctx) {
+      m_timer->add_event_after(m_update_sync_point_interval,
+                               m_update_sync_ctx);
+    }
+  } else {
     send_flush_sync_point();
   }
 }
@@ -214,8 +328,7 @@ void ImageCopyRequest<I>::send_flush_sync_point() {
     m_sync_point->object_number = boost::none;
   }
 
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": sync_point=" << *m_sync_point << dendl;
+  dout(20) << ": sync_point=" << *m_sync_point << dendl;
 
   bufferlist client_data_bl;
   librbd::journal::ClientData client_data(m_client_meta_copy);
@@ -229,14 +342,13 @@ void ImageCopyRequest<I>::send_flush_sync_point() {
 
 template <typename I>
 void ImageCopyRequest<I>::handle_flush_sync_point(int r) {
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": r=" << r << dendl;
+  dout(20) << ": r=" << r << dendl;
 
   if (r < 0) {
     *m_client_meta = m_client_meta_copy;
 
-    lderr(cct) << ": failed to update client data: " << cpp_strerror(r)
-               << dendl;
+    derr << ": failed to update client data: " << cpp_strerror(r)
+         << dendl;
     finish(r);
     return;
   }
@@ -244,18 +356,8 @@ void ImageCopyRequest<I>::handle_flush_sync_point(int r) {
   finish(0);
 }
 
-template <typename I>
-void ImageCopyRequest<I>::finish(int r) {
-  CephContext *cct = m_local_image_ctx->cct;
-  ldout(cct, 20) << ": r=" << r << dendl;
-
-  m_on_finish->complete(r);
-  delete this;
-}
-
 template <typename I>
 int ImageCopyRequest<I>::compute_snap_map() {
-  CephContext *cct = m_local_image_ctx->cct;
 
   librados::snap_t snap_id_start = 0;
   librados::snap_t snap_id_end;
@@ -263,8 +365,8 @@ int ImageCopyRequest<I>::compute_snap_map() {
     RWLock::RLocker snap_locker(m_remote_image_ctx->snap_lock);
     snap_id_end = m_remote_image_ctx->get_snap_id(m_sync_point->snap_name);
     if (snap_id_end == CEPH_NOSNAP) {
-      lderr(cct) << ": failed to locate snapshot: "
-                 << m_sync_point->snap_name << dendl;
+      derr << ": failed to locate snapshot: "
+           << m_sync_point->snap_name << dendl;
       return -ENOENT;
     }
 
@@ -272,8 +374,8 @@ int ImageCopyRequest<I>::compute_snap_map() {
       snap_id_start = m_remote_image_ctx->get_snap_id(
         m_sync_point->from_snap_name);
       if (snap_id_start == CEPH_NOSNAP) {
-        lderr(cct) << ": failed to locate from snapshot: "
-                   << m_sync_point->from_snap_name << dendl;
+        derr << ": failed to locate from snapshot: "
+             << m_sync_point->from_snap_name << dendl;
         return -ENOENT;
       }
     }
@@ -293,7 +395,7 @@ int ImageCopyRequest<I>::compute_snap_map() {
   }
 
   if (m_snap_map.empty()) {
-    lderr(cct) << ": failed to map snapshots within boundary" << dendl;
+    derr << ": failed to map snapshots within boundary" << dendl;
     return -EINVAL;
   }
 
